@@ -1,6 +1,6 @@
 /* canvas-query-join.c
  *
- * Copyright (C) 2002 Vivien Malerba
+ * Copyright (C) 2002 - 2003 Vivien Malerba
  * Copyright (C) 2002 Fernando Martins
  *
  * This program is free software; you can redistribute it and/or
@@ -22,6 +22,7 @@
 #include "canvas-query-join.h"
 #include "canvas-field.h"
 #include "relship-view.h"
+#include "conds-view.h"
 
 static void canvas_query_join_class_init (CanvasQueryJoinClass * class);
 static void canvas_query_join_init (CanvasQueryJoin * item);
@@ -131,7 +132,6 @@ enter_notify_cb (CanvasQueryJoin * cqj, gpointer data)
 	while (list) {
 		gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (list->data));
 		gnome_canvas_item_set (GNOME_CANVAS_ITEM (list->data),
-				       "width_units", 3.,
 				       "fill_color", "red",
 				       NULL);
 		list = g_slist_next (list);
@@ -201,7 +201,6 @@ leave_notify_cb (CanvasQueryJoin * cqj, gpointer data)
 	while (list) {
 		gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (list->data));
 		gnome_canvas_item_set (GNOME_CANVAS_ITEM (list->data),
-				       "width_units", 1.5,
 				       "fill_color", "black",
 				       NULL);
 		list = g_slist_next (list);
@@ -503,8 +502,18 @@ redraw_view_contents (CanvasQueryJoin *cqj)
 
 static GObject *cond_get_object_ref (QueryCond *cond, gboolean left_op);
 
+static gint get_point_rect_area (double x1, double y1, double x2, double y2, double x, double y);
 static int cond_item_event(GnomeCanvasItem *center, GdkEvent *event, QueryCond *cond);
-/* this is a recursive function */
+
+
+/* Draws a representation of the QueryCond associated to a QueryJoin: between the two representations
+ * of the QueryViews.
+ *
+ * It is a list of lines joining the fields of the QueryViews if we can represent the
+ * condition as: field1 = field2 [AND field3 = field4 [AND...]]
+ *
+ * Otherwise it is a synthetic view of the join condition
+ */
 static void
 draw_query_cond (CanvasQueryJoin *cqj, QueryCond *cond)
 {
@@ -807,14 +816,197 @@ draw_query_cond (CanvasQueryJoin *cqj, QueryCond *cond)
 		}
 	}
 	else {
-		if (!drawn) {
-			/* TODO */
-			g_print ("Complete non equi joins drawing\n");
+		if (!drawn) { /* We simply draw a thick line with a clrcle in the middle */
+			/* we don't care about which QueryView is ant_view and which is suc_view */
+			double x1a, y1a, x2a, y2a; /* boundaries of ant_view */
+			double x1s, y1s, x2s, y2s; /* boundaries of suc_view */
+			double xca, yca, xcs, ycs; /* center points */
+			double x1, y1, x2, y2, lx1, ly1, lx2, ly2; /* lines to draw */
+			double Xo = 3., Yo = 3.;
+			GnomeCanvasPoints *points;
+			GnomeCanvasItem *item;
+			gint ara, ars; /* areas of the rectangle, see get_point_rect_area() */
+
+			gnome_canvas_item_get_bounds (cqj->ant_cview, &x1a, &y1a, &x2a, &y2a);
+			gnome_canvas_item_get_bounds (cqj->suc_cview, &x1s, &y1s, &x2s, &y2s);
+			
+			xca = (x1a + x2a)/2.;
+			yca = (y1a + y2a)/2.;
+			xcs = (x1s + x2s)/2.;
+			ycs = (y1s + y2s)/2.;
+
+			ara = get_point_rect_area (x1a, y1a, x2a, y2a, xcs, ycs);
+			ars = get_point_rect_area (x1s, y1s, x2s, y2s, xca, yca);
+			
+			/* line for ant_view */
+			switch (ara) {
+			case 1:
+				x1 = x1a;
+				x2 = x2a;
+				y1 = y2 = y1a - Yo;
+				break;
+			case 2:
+				x1 = x2 = x2a + Xo;
+				y1 = y1a;
+				y2 = y2a;
+				break;
+			case 3:
+				x1 = x1a;
+				x2 = x2a;
+				y1 = y2 = y2a + Yo;
+				break;
+			case 4:
+				x1 = x2 = x1a - Yo;
+				y1 = y1a;
+				y2 = y2a;
+				break;
+			default:
+				x1 = y1 = x2 = y2 = 0;
+				break;
+			}
+			lx1 = (x1 + x2)/2.;
+			ly1 = (y1 + y2)/2.;
+
+			points = gnome_canvas_points_new (2);
+			points->coords[0] = x1;
+			points->coords[1] = y1;
+			points->coords[2] = x2;
+			points->coords[3] = y2;
+			item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (cqj),
+						      GNOME_TYPE_CANVAS_LINE,
+						      "points", points,
+						      "fill_color", "black",
+						      "width_units", 1.5,
+						      "cap_style", GDK_CAP_ROUND,
+						      "smooth", TRUE,
+						      NULL);
+
+			gnome_canvas_item_lower_to_bottom (item);
+			cqj->items = g_slist_append (cqj->items, item);
+			gnome_canvas_points_free (points);
+			g_signal_connect (G_OBJECT (item),"event",
+					  G_CALLBACK (cond_item_event), cond);
+
+			/* line for suc_view */
+			switch (ars) {
+			case 1:
+				x1 = x1s;
+				x2 = x2s;
+				y1 = y2 = y1s - Yo;
+				break;
+			case 2:
+				x1 = x2 = x2s + Xo;
+				y1 = y1s;
+				y2 = y2s;
+				break;
+			case 3:
+				x1 = x1s;
+				x2 = x2s;
+				y1 = y2 = y2s + Yo;
+				break;
+			case 4:
+				x1 = x2 = x1s - Xo;
+				y1 = y1s;
+				y2 = y2s;
+				break;
+			default:
+				x1 = y1 = x2 = y2 = 0;
+				break;
+			}
+			lx2 = (x1 + x2)/2.;
+			ly2 = (y1 + y2)/2.;
+
+			points = gnome_canvas_points_new (2);
+			points->coords[0] = x1;
+			points->coords[1] = y1;
+			points->coords[2] = x2;
+			points->coords[3] = y2;
+			item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (cqj),
+						      GNOME_TYPE_CANVAS_LINE,
+						      "points", points,
+						      "fill_color", "black",
+						      "width_units", 1.5,
+						      "cap_style", GDK_CAP_ROUND,
+						      "smooth", TRUE,
+						      NULL);
+
+			gnome_canvas_item_lower_to_bottom (item);
+			cqj->items = g_slist_append (cqj->items, item);
+			gnome_canvas_points_free (points);
+			g_signal_connect (G_OBJECT (item),"event",
+					  G_CALLBACK (cond_item_event), cond);
+			
+			/* junction line */
+			points = gnome_canvas_points_new (2);
+			points->coords[0] = lx1;
+			points->coords[1] = ly1;
+			points->coords[2] = lx2;
+			points->coords[3] = ly2;
+			item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (cqj),
+						      GNOME_TYPE_CANVAS_LINE,
+						      "points", points,
+						      "fill_color", "black",
+						      "width_units", 1.5,
+						      "cap_style", GDK_CAP_ROUND,
+						      "smooth", TRUE,
+						      NULL);
+
+			gnome_canvas_item_lower_to_bottom (item);
+			cqj->items = g_slist_append (cqj->items, item);
+			gnome_canvas_points_free (points);
+			g_signal_connect (G_OBJECT (item),"event",
+					  G_CALLBACK (cond_item_event), cond);
+
 		}
 	}
 }
 
-/* returns the DbField or QueryField from a QueryCond if it is and EQUAL condition */
+/* Returns the area defined from the rectangle where the point (x,y) is:
+ *     \  1   / 
+ *      \____/    *
+ *      |\  /| 
+ *   4  |    | 2
+ *      |/__\|
+ *      /    \
+ *     /   3  \
+ *    D1       D2
+ *
+ *    D1: y = a1 x + b1
+ *    D2: y = a2 x + b2
+ */
+static gint
+get_point_rect_area (double x1, double y1, double x2, double y2, double x, double y)
+{
+	double a1, a2, b1, b2, img1, img2;
+	gint retval;
+	g_return_val_if_fail (x1 != x2, -1);
+	
+	a1 = (y2 - y1) / (x1 - x2);
+	a2 = -a1;
+	
+	b1 = y2 - a1 * x1;
+	b2 = y1 - a2 * x1;
+
+	img1 = a1 * x + b1;
+	img2 = a2 * x + b2;
+
+	if (y >= img1) {
+		if (y < img2)
+			retval = 2;
+		else 
+			retval = 3;
+	}
+	else {
+		if (y < img2)
+			retval = 1;
+		else
+			retval = 4;
+	}
+
+	return retval;
+}
+
+/* returns the DbField or QueryField from a QueryCond if it is an EQUAL condition */
 static GObject *
 cond_get_object_ref (QueryCond *cond, gboolean left_op)
 {
@@ -948,7 +1140,7 @@ edit_join_props_cb (GtkWidget *button, CanvasQueryJoin *cqj)
 		gchar *name1, *name2;
 		GtkDialog *dlg;
 		GtkWidget *label, *frame, *rb, *rb2, *vbox, *cb, *table;
-		GtkWidget *window;
+		GtkWidget *window, *nb, *vb;
 	
 		if (cqj->query == QUERY (cqj->query->conf->top_query))
 			is_relation = TRUE;
@@ -963,7 +1155,7 @@ edit_join_props_cb (GtkWidget *button, CanvasQueryJoin *cqj)
 						  GTK_TYPE_WINDOW);
 		dlg = GTK_DIALOG (gtk_dialog_new_with_buttons (str, GTK_WINDOW (window), 0, 
 							       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-							       NULL));
+							       NULL));		
 
 		/* A label for title */
 		name1 = query_view_get_textual (query_join_get_ant_view (cqj->join));
@@ -979,10 +1171,17 @@ edit_join_props_cb (GtkWidget *button, CanvasQueryJoin *cqj)
 		label = gtk_label_new (str);
 		gtk_box_pack_start (GTK_BOX (dlg->vbox), label, FALSE, FALSE, GNOME_PAD/2.);
 
+		/* Notebook */
+		nb = gtk_notebook_new ();
+		gtk_box_pack_start (GTK_BOX (dlg->vbox), nb, TRUE, TRUE, GNOME_PAD/2.);
+
 		/* Type of join */
+		label = gtk_label_new (_("Basic properties"));
+		vb = gtk_vbox_new (FALSE, GNOME_PAD/2.);
+		gtk_notebook_append_page (GTK_NOTEBOOK (nb), vb, label);
 		if (!is_relation) {
 			frame = gtk_frame_new (_("Join type"));
-			gtk_box_pack_start (GTK_BOX (dlg->vbox), frame, FALSE, FALSE, GNOME_PAD/2.);
+			gtk_box_pack_start (GTK_BOX (vb), frame, FALSE, FALSE, GNOME_PAD/2.);
 			
 			table = gtk_table_new (4, 2, FALSE);
 			gtk_container_set_border_width (GTK_CONTAINER (table),GNOME_PAD/2.);
@@ -1032,14 +1231,14 @@ edit_join_props_cb (GtkWidget *button, CanvasQueryJoin *cqj)
 		if (is_relation) {
 			/* Referential integrity */
 			frame = gtk_frame_new (_("Referential integrity"));
-			gtk_box_pack_start (GTK_BOX (dlg->vbox), frame, FALSE, FALSE, GNOME_PAD/2.);
+			gtk_box_pack_start (GTK_BOX (vb), frame, FALSE, FALSE, GNOME_PAD/2.);
 			gtk_widget_set_sensitive (frame, FALSE);
 			
 			vbox = gtk_vbox_new (TRUE, GNOME_PAD/2.);
 			gtk_container_set_border_width (GTK_CONTAINER (vbox),GNOME_PAD/2.);
 			gtk_container_add (GTK_CONTAINER (frame), vbox);
 			
-			cb = gtk_check_button_new_with_label (_("Apply"));
+			cb = gtk_check_button_new_with_label (_("Enforce referential integrity"));
 			gtk_box_pack_start (GTK_BOX (vbox), cb, FALSE, FALSE, 0);
 			
 			cb = gtk_check_button_new_with_label (_("Update in cascade"));
@@ -1047,50 +1246,55 @@ edit_join_props_cb (GtkWidget *button, CanvasQueryJoin *cqj)
 			
 			cb = gtk_check_button_new_with_label (_("Delete in cascade"));
 			gtk_box_pack_start (GTK_BOX (vbox), cb, FALSE, FALSE, 0);
+
+
+			/* Cardinality */
+			frame = gtk_frame_new (_("Join cardinality"));
+			gtk_box_pack_start (GTK_BOX (vb), frame, FALSE, FALSE, GNOME_PAD/2.);
+			
+			vbox = gtk_vbox_new (TRUE, GNOME_PAD/2.);
+			gtk_container_set_border_width (GTK_CONTAINER (vbox),GNOME_PAD/2.);
+			gtk_container_add (GTK_CONTAINER (frame), vbox);
+			
+			rb = gtk_radio_button_new_with_label (NULL, _("One to One"));
+			gtk_box_pack_start (GTK_BOX (vbox), rb, FALSE, FALSE, 0);
+			g_object_set_data (G_OBJECT (dlg), card_prop_name (QUERY_JOIN_1_1), rb);
+			g_object_set_data (G_OBJECT (rb), "card", GINT_TO_POINTER (QUERY_JOIN_1_1));
+			g_signal_connect (G_OBJECT (rb), "toggled",
+					  G_CALLBACK (change_join_card_cb), cqj->join);
+			
+			
+			rb2 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (rb), 
+									   _("One to Many"));
+			gtk_box_pack_start (GTK_BOX (vbox), rb2, FALSE, FALSE, 0);
+			g_object_set_data (G_OBJECT (dlg), card_prop_name (QUERY_JOIN_1_N), rb2);
+			g_object_set_data (G_OBJECT (rb2), "card", GINT_TO_POINTER (QUERY_JOIN_1_N));
+			g_signal_connect (G_OBJECT (rb2), "toggled",
+					  G_CALLBACK (change_join_card_cb), cqj->join);
+			
+			rb = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (rb), 
+									  _("Many to One"));
+			gtk_box_pack_start (GTK_BOX (vbox), rb, FALSE, FALSE, 0);
+			g_object_set_data (G_OBJECT (dlg), card_prop_name (QUERY_JOIN_N_1), rb);
+			g_object_set_data (G_OBJECT (rb), "card", GINT_TO_POINTER (QUERY_JOIN_N_1));
+			g_signal_connect (G_OBJECT (rb), "toggled",
+					  G_CALLBACK (change_join_card_cb), cqj->join);
+			
+			
+			rb2 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (rb), 
+									   _("Undefined"));
+			gtk_box_pack_start (GTK_BOX (vbox), rb2, FALSE, FALSE, 0);
+			g_object_set_data (G_OBJECT (dlg), card_prop_name (QUERY_JOIN_UNDEFINED), rb2);
+			g_object_set_data (G_OBJECT (rb2), "card", GINT_TO_POINTER (QUERY_JOIN_UNDEFINED));
+			g_signal_connect (G_OBJECT (rb2), "toggled",
+					  G_CALLBACK (change_join_card_cb), cqj->join);
+			join_card_changed_cb (cqj->join, dlg);
 		}
 
-		/* Cardinality */
-		frame = gtk_frame_new (_("Join cardinality"));
-		gtk_box_pack_start (GTK_BOX (dlg->vbox), frame, FALSE, FALSE, GNOME_PAD/2.);
-		
-		vbox = gtk_vbox_new (TRUE, GNOME_PAD/2.);
-		gtk_container_set_border_width (GTK_CONTAINER (vbox),GNOME_PAD/2.);
-		gtk_container_add (GTK_CONTAINER (frame), vbox);
-
-		rb = gtk_radio_button_new_with_label (NULL, _("One to One"));
-		gtk_box_pack_start (GTK_BOX (vbox), rb, FALSE, FALSE, 0);
-		g_object_set_data (G_OBJECT (dlg), card_prop_name (QUERY_JOIN_1_1), rb);
-		g_object_set_data (G_OBJECT (rb), "card", GINT_TO_POINTER (QUERY_JOIN_1_1));
-		g_signal_connect (G_OBJECT (rb), "toggled",
-				  G_CALLBACK (change_join_card_cb), cqj->join);
-
-
-		rb2 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (rb), 
-								  _("One to Many"));
-		gtk_box_pack_start (GTK_BOX (vbox), rb2, FALSE, FALSE, 0);
-		g_object_set_data (G_OBJECT (dlg), card_prop_name (QUERY_JOIN_1_N), rb2);
-		g_object_set_data (G_OBJECT (rb2), "card", GINT_TO_POINTER (QUERY_JOIN_1_N));
-		g_signal_connect (G_OBJECT (rb2), "toggled",
-				  G_CALLBACK (change_join_card_cb), cqj->join);
-
-		rb = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (rb), 
-								  _("Many to One"));
-		gtk_box_pack_start (GTK_BOX (vbox), rb, FALSE, FALSE, 0);
-		g_object_set_data (G_OBJECT (dlg), card_prop_name (QUERY_JOIN_N_1), rb);
-		g_object_set_data (G_OBJECT (rb), "card", GINT_TO_POINTER (QUERY_JOIN_N_1));
-		g_signal_connect (G_OBJECT (rb), "toggled",
-				  G_CALLBACK (change_join_card_cb), cqj->join);
-
-
-		rb2 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (rb), 
-								   _("Undefined"));
-		gtk_box_pack_start (GTK_BOX (vbox), rb2, FALSE, FALSE, 0);
-		g_object_set_data (G_OBJECT (dlg), card_prop_name (QUERY_JOIN_UNDEFINED), rb2);
-		g_object_set_data (G_OBJECT (rb2), "card", GINT_TO_POINTER (QUERY_JOIN_UNDEFINED));
-		g_signal_connect (G_OBJECT (rb2), "toggled",
-				  G_CALLBACK (change_join_card_cb), cqj->join);
-		join_card_changed_cb (cqj->join, dlg);
-
+		/* Associated QueryCond edition */
+		label = gtk_label_new (_("Advanced"));
+		vb = conds_view_new_in_sw (G_OBJECT (cqj->join));
+		gtk_notebook_append_page (GTK_NOTEBOOK (nb), vb, label);
 
 		/* Signals */
 		g_signal_connect (G_OBJECT (dlg), "response",
