@@ -1214,8 +1214,10 @@ struct ts_extension {
 #define TS_EXTENSION(de) ((struct ts_extension *) (DATA_ENTRY (de)->extension))
 
 void ts_free_extension (DataEntry *de) {
-	g_free (de->extension);
-	de->extension = NULL;
+	if (de->extension != NULL) {
+		g_free (de->extension);
+		de->extension = NULL;
+	}
 }
 
 
