@@ -719,131 +719,154 @@ database_load_tables (Database * db, ServerAccess * srv)
 					 GDA_CONNECTION_SCHEMA_TABLES, NULL); 
 
 	if (rst) {
-		total = gda_data_model_get_n_rows (rst);
-		now = 0;		
-		while (now < total) {
-			/* table name */
-			value = gda_data_model_get_value_at (rst, 0, now);
-			str = gda_value_stringify (value);	
-			table = database_find_table_from_name (db, str);
-			if (!table) {	/* the table doesn't already exist, create it */
-				table = DB_TABLE (db_table_new ());
-				table->name = str;
-				str = NULL;
-
-				g_signal_connect (G_OBJECT (table), "load_fault",
-						    G_CALLBACK (database_cb), db);
-				g_signal_connect (G_OBJECT (table), "field_created",
-						    G_CALLBACK (database_catch_field_create_cb),
-						    db);
-				g_signal_connect (G_OBJECT (table), "field_dropped",
-						    G_CALLBACK (database_catch_field_drop_cb),
-						    db);
-				db->tables = g_slist_append (db->tables, table);
-				created = TRUE;
-			}
-			else {
-				created = FALSE;
-			}
-			if (str)
-				g_free (str);
-
-			/* (re)computes the comments */
-			value = gda_data_model_get_value_at (rst, 2, now);
-			if (value && !gda_value_is_null (value)) {
-				str = gda_value_stringify (value);
-				db_table_set_comments (table, NULL, str, FALSE);
-				g_free (str);
-			}
-
-			/* (re)computes the owner */
-			value = gda_data_model_get_value_at (rst, 1, now);
-			if (value && !gda_value_is_null (value)) {
-				GSList *list;
-				str = gda_value_stringify (value);
-				list = find_user_name (db, str);
-				table->owner = (gchar *) (list->data);
-				g_free (str);
-			}
-
-			if (created) {	/* to emit signals now because then owner and comments filled */
-#ifdef debug_signal
-				g_print (">> 'TABLE_CREATED' from database_load_tables\n");
-#endif
-				g_signal_emit (G_OBJECT (db), database_signals[TABLE_CREATED], 0, table);
-#ifdef debug_signal
-				g_print ("<< 'TABLE_CREATED' from database_load_tables\n");
-#endif
-			}
-
-			db_table_load_fields (table, srv, db);
-			if (created) {
-#ifdef debug_signal
-				g_print (">> 'TABLE_CREATED_FILLED' from database_load_tables\n");
-#endif
-				g_signal_emit (G_OBJECT (db), database_signals[TABLE_CREATED_FILLED], 0,
-					       table);
-#ifdef debug_signal
-				g_print ("<< 'TABLE_CREATED_FILLED' from database_load_tables\n");
-#endif
-			}
-			/* mark the table as updated */
-			((DbItem *) table)->updated = TRUE;
-
-			g_signal_emit_by_name (G_OBJECT (db), "progress",
-					       "TABLES",
-					       now, total);
-			now++;
-		}
-		g_object_unref (G_OBJECT (rst));
-
-		if (!(db->is_fault)) {
-			/***************************************************************/
-			/* 2nd step: re-compute the parents table list for every table */
-			/***************************************************************/
-			list = db->tables;
-			while (list) {
-				if ((DB_TABLE (list->data)->is_view ==  FALSE) 
-				    && (DB_ITEM (list->data)->updated == TRUE)) {
-					GdaParameterList *paramlist;
-					GdaParameter *param;
-					
-					paramlist = gda_parameter_list_new ();
-					param = gda_parameter_new_string ("name", DB_TABLE (list->data)->name);
-					gda_parameter_list_add_parameter (paramlist, param);
-					rst = gda_connection_get_schema (GDA_CONNECTION (srv->cnc),
-									 GDA_CONNECTION_SCHEMA_PARENT_TABLES, 
-									 paramlist); 
-					gda_parameter_list_free (paramlist);
+		if (server_resultset_check_data_model (rst, 4, 
+						       GDA_VALUE_TYPE_STRING, 
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING,
+						       GDA_VALUE_TYPE_STRING)) {
+			total = gda_data_model_get_n_rows (rst);
+			now = 0;		
+			while (now < total) {
+				/* table name */
+				value = gda_data_model_get_value_at (rst, 0, now);
+				str = gda_value_stringify (value);	
+				table = database_find_table_from_name (db, str);
+				if (!table) {	/* the table doesn't already exist, create it */
+					table = DB_TABLE (db_table_new ());
+					table->name = str;
+					str = NULL;
 					
+					g_signal_connect (G_OBJECT (table), "load_fault",
+							  G_CALLBACK (database_cb), db);
+					g_signal_connect (G_OBJECT (table), "field_created",
+							  G_CALLBACK (database_catch_field_create_cb),
+							  db);
+					g_signal_connect (G_OBJECT (table), "field_dropped",
+							  G_CALLBACK (database_catch_field_drop_cb),
+							  db);
+					db->tables = g_slist_append (db->tables, table);
+					created = TRUE;
+				}
+				else {
+					created = FALSE;
+				}
+				if (str)
+					g_free (str);
+				
+				/* (re)computes the comments */
+				value = gda_data_model_get_value_at (rst, 2, now);
+				if (value && !gda_value_is_null (value)) {
+					str = gda_value_stringify (value);
+					db_table_set_comments (table, NULL, str, FALSE);
+					g_free (str);
+				}
 
+				/* (re)computes the owner */
+				value = gda_data_model_get_value_at (rst, 1, now);
+				if (value && !gda_value_is_null (value)) {
+					GSList *list;
+					str = gda_value_stringify (value);
+					list = find_user_name (db, str);
+					table->owner = (gchar *) (list->data);
+					g_free (str);
+				}
+				
+				if (created) {	/* to emit signals now because then owner and comments filled */
+#ifdef debug_signal
+					g_print (">> 'TABLE_CREATED' from database_load_tables\n");
+#endif
+					g_signal_emit (G_OBJECT (db), database_signals[TABLE_CREATED], 0, table);
+#ifdef debug_signal
+					g_print ("<< 'TABLE_CREATED' from database_load_tables\n");
+#endif
+				}
+				
+				db_table_load_fields (table, srv, db);
+				if (created) {
+#ifdef debug_signal
+					g_print (">> 'TABLE_CREATED_FILLED' from database_load_tables\n");
+#endif
+					g_signal_emit (G_OBJECT (db), database_signals[TABLE_CREATED_FILLED], 0,
+						       table);
+#ifdef debug_signal
+					g_print ("<< 'TABLE_CREATED_FILLED' from database_load_tables\n");
+#endif
+				}
+				/* mark the table as updated */
+				((DbItem *) table)->updated = TRUE;
+				
+				g_signal_emit_by_name (G_OBJECT (db), "progress",
+						       "TABLES",
+						       now, total);
+				now++;
+			}
+			g_object_unref (G_OBJECT (rst));
 
-					g_slist_free (DB_TABLE (list->data)->parents);
-					DB_TABLE (list->data)->parents = NULL;
-					
-					if (rst) {
-						guint now2, total2;
+			if (!(db->is_fault)) {
+				/***************************************************************/
+				/* 2nd step: re-compute the parents table list for every table */
+				/***************************************************************/
+				list = db->tables;
+				while (list) {
+					if ((DB_TABLE (list->data)->is_view ==  FALSE) 
+					    && (DB_ITEM (list->data)->updated == TRUE)) {
+						GdaParameterList *paramlist;
+						GdaParameter *param;
+						
+						paramlist = gda_parameter_list_new ();
+						param = gda_parameter_new_string ("name", DB_TABLE (list->data)->name);
+						gda_parameter_list_add_parameter (paramlist, param);
+						rst = gda_connection_get_schema (GDA_CONNECTION (srv->cnc),
+										 GDA_CONNECTION_SCHEMA_PARENT_TABLES, 
+										 paramlist); 
+						gda_parameter_list_free (paramlist);
+						
+						
+						
+						g_slist_free (DB_TABLE (list->data)->parents);
+						DB_TABLE (list->data)->parents = NULL;
+						
+						if (rst) {
+							if (server_resultset_check_data_model (rst, 2, 
+											       GDA_VALUE_TYPE_STRING, 
+											       GDA_VALUE_TYPE_INTEGER)) {
+								guint now2, total2;
+								
+								list2 = NULL;
+								total2 = gda_data_model_get_n_rows (rst);
+								now2 = 0;
+								while (now2 < total2) {
+									value = gda_data_model_get_value_at (rst, 0, now2);
+									str = gda_value_stringify (value);
+									table = database_find_table_from_name (db, str);
+									g_free (str);
+									
+									if (table && (table != DB_TABLE (list->data)))
+										list2 = g_slist_append (list2, table);
+									now2++;
+								}
+							}
+							else
+								g_warning ("The provider has returned the wrong \n"
+									   "number and/or types of data\n"
+									   "for the PARENT_TABLES schema description.\n"
+									   "Please contact the provider's maintainer "
+									   "and report the problem.");
 
-						list2 = NULL;
-						total2 = gda_data_model_get_n_rows (rst);
-						now2 = 0;
-						while (now2 < total2) {
-							value = gda_data_model_get_value_at (rst, 0, now2);
-							str = gda_value_stringify (value);
-							table = database_find_table_from_name (db, str);
-							g_free (str);
-
-							if (table && (table != DB_TABLE (list->data)))
-								list2 = g_slist_append (list2, table);
-							now2++;
+							g_object_unref (G_OBJECT (rst));
 						}
-						g_object_unref (G_OBJECT (rst));
 						DB_TABLE (list->data)->	parents = list2;
 					}
+					list = g_slist_next (list);
 				}
-				list = g_slist_next (list);
 			}
 		}
+		else 
+			g_warning ("The provider has returned the wrong \n"
+				   "number and/or types of data\n"
+				   "for the TABLES schema description.\n"
+				   "Please contact the provider's maintainer "
+				   "and report the problem.");
 	}
 	else {
 		/* An error occured, the db emits the fault signal */
@@ -882,86 +905,99 @@ database_load_views (Database * db, ServerAccess * srv)
 					 GDA_CONNECTION_SCHEMA_VIEWS, NULL); 
 
 	if (rst) {
-		total = gda_data_model_get_n_rows (rst);
-		now = 0;
+		if (server_resultset_check_data_model (rst, 4, 
+						       GDA_VALUE_TYPE_STRING, 
+						       GDA_VALUE_TYPE_STRING, 
+						       GDA_VALUE_TYPE_STRING, 
+						       GDA_VALUE_TYPE_STRING)) {
+			total = gda_data_model_get_n_rows (rst);
+			now = 0;
+			
+			while (now < total) {
+				/* view name */
+				value = gda_data_model_get_value_at (rst, 0, now);
+				str = gda_value_stringify (value);	
+				table = database_find_table_from_name (db, str);
+				if (!table) {	/* the table doesn't already exist, create it */
+					table = DB_TABLE (db_table_new ());
+					table->name = str;
+					str = NULL;
+					
+					g_signal_connect (G_OBJECT (table), "load_fault",
+							  G_CALLBACK (database_cb), db);
+					g_signal_connect (G_OBJECT (table), "field_created",
+							  G_CALLBACK (database_catch_field_create_cb),
+							  db);
+					g_signal_connect (G_OBJECT (table), "field_dropped",
+							  G_CALLBACK (database_catch_field_drop_cb),
+							  db);
+					db->tables = g_slist_append (db->tables, table);
+					created = TRUE;
+				}
+				else {
+					created = FALSE;
+				}
+				if (str)
+					g_free (str);
 
-		while (now < total) {
-			/* view name */
-			value = gda_data_model_get_value_at (rst, 0, now);
-			str = gda_value_stringify (value);	
-			table = database_find_table_from_name (db, str);
-			if (!table) {	/* the table doesn't already exist, create it */
-				table = DB_TABLE (db_table_new ());
-				table->name = str;
-				str = NULL;
-
-				g_signal_connect (G_OBJECT (table), "load_fault",
-						    G_CALLBACK (database_cb), db);
-				g_signal_connect (G_OBJECT (table), "field_created",
-						    G_CALLBACK (database_catch_field_create_cb),
-						    db);
-				g_signal_connect (G_OBJECT (table), "field_dropped",
-						    G_CALLBACK (database_catch_field_drop_cb),
-						    db);
-				db->tables = g_slist_append (db->tables, table);
-				created = TRUE;
-			}
-			else {
-				created = FALSE;
-			}
-			if (str)
-				g_free (str);
-
-			/* set it here because otherwise if an object was a table and 
-			   becomes a view, this won't be set correctly */
-			table->is_view = TRUE;
-
-			/* (re)computes the comments */
-			value = gda_data_model_get_value_at (rst, 2, now);
-			if (!gda_value_is_null (value)) {
-				str = gda_value_stringify (value);
-				db_table_set_comments (table, NULL, str, FALSE);
-				g_free (str);
-			}
-
-			/* (re)computes the owner */
-			value = gda_data_model_get_value_at (rst, 1, now);
-			if (!gda_value_is_null (value)) {
-				GSList *list;
-				str = gda_value_stringify (value);
-				list = find_user_name (db, str);
-				table->owner = (gchar *) (list->data);
-				g_free (str);
-			}
-
-			if (created) {	/* to emit signals now because then owner and comments filled */
+				/* set it here because otherwise if an object was a table and 
+				   becomes a view, this won't be set correctly */
+				table->is_view = TRUE;
+				
+				/* (re)computes the comments */
+				value = gda_data_model_get_value_at (rst, 2, now);
+				if (!gda_value_is_null (value)) {
+					str = gda_value_stringify (value);
+					db_table_set_comments (table, NULL, str, FALSE);
+					g_free (str);
+				}
+				
+				/* (re)computes the owner */
+				value = gda_data_model_get_value_at (rst, 1, now);
+				if (!gda_value_is_null (value)) {
+					GSList *list;
+					str = gda_value_stringify (value);
+					list = find_user_name (db, str);
+					table->owner = (gchar *) (list->data);
+					g_free (str);
+				}
+				
+				if (created) {	/* to emit signals now because then owner and comments filled */
 #ifdef debug_signal
-				g_print (">> 'TABLE_CREATED' from database_load_tables\n");
+					g_print (">> 'TABLE_CREATED' from database_load_tables\n");
 #endif
-				g_signal_emit (G_OBJECT (db), database_signals[TABLE_CREATED], 0, table);
+					g_signal_emit (G_OBJECT (db), database_signals[TABLE_CREATED], 0, table);
 #ifdef debug_signal
-				g_print ("<< 'TABLE_CREATED' from database_load_tables\n");
+					g_print ("<< 'TABLE_CREATED' from database_load_tables\n");
 #endif
+				}
+				db_table_load_fields (table, srv, db);
+				if (created) {
+#ifdef debug_signal
+					g_print (">> 'TABLE_CREATED_FILLED' from database_load_tables\n");
+#endif
+					g_signal_emit (G_OBJECT (db), database_signals[TABLE_CREATED_FILLED], 0,
+						       table);
+#ifdef debug_signal
+					g_print ("<< 'TABLE_CREATED_FILLED' from database_load_tables\n");
+#endif
+				}
+				/* mark the table as updated */
+				((DbItem *) table)->updated = TRUE;
+				
+				g_signal_emit_by_name (G_OBJECT (db), "progress",
+						       "VIEWS",
+						       now, total);
+				now++;
 			}
-			db_table_load_fields (table, srv, db);
-			if (created) {
-#ifdef debug_signal
-				g_print (">> 'TABLE_CREATED_FILLED' from database_load_tables\n");
-#endif
-				g_signal_emit (G_OBJECT (db), database_signals[TABLE_CREATED_FILLED], 0,
-					       table);
-#ifdef debug_signal
-				g_print ("<< 'TABLE_CREATED_FILLED' from database_load_tables\n");
-#endif
-			}
-			/* mark the table as updated */
-			((DbItem *) table)->updated = TRUE;
-
-			g_signal_emit_by_name (G_OBJECT (db), "progress",
-						 "VIEWS",
-						 now, total);
-			now++;
 		}
+		else
+			g_warning ("The provider has returned the wrong \n"
+				   "number and/or types of data\n"
+				   "for the VIEWS schema description.\n"
+				   "Please contact the provider's maintainer "
+				   "and report the problem.");
+			
 		g_object_unref (G_OBJECT (rst));
 	}
 	g_signal_emit_by_name (G_OBJECT (db), "progress", NULL, 0, 0);
@@ -1033,63 +1069,79 @@ database_load_sequences (Database * db, ServerAccess * srv)
 					GDA_CONNECTION_SCHEMA_SEQUENCES, NULL); 
 
 	if (rs) {
-		total = gda_data_model_get_n_rows (rs);
-		now = 0;
-		while (now < total) {
-			value = gda_data_model_get_value_at (rs, 0, now);
-			str = gda_value_stringify (value);
-			created = FALSE;
-			seq = database_find_sequence_by_name (db, str);
-			if (!seq) {
-				seq = DB_SEQUENCE (db_sequence_new ());
-				seq->name = str;
-				str = NULL;
-				g_signal_connect (G_OBJECT (seq), "load_fault",
-						    G_CALLBACK (database_cb), db);
-				db->sequences = g_slist_append (db->sequences, seq);
-				created = TRUE;
-			}
-			if (str)
-				g_free (str);
-
-			/* (re)computes the comments */
-			value = gda_data_model_get_value_at (rs, 2, now);
-			if (!gda_value_is_null (value)) {
-				str = gda_value_stringify (value);	/* str allocated */
-				if (seq->comments)
-					g_free (seq->comments);
-				seq->comments = str;
-			}
-
-			/* (re)computes the owner */
-			value = gda_data_model_get_value_at (rs, 1, now);
-			if (!gda_value_is_null (value)) {
-				GSList *list;
+		/* check that the provider returns the correct columns */
+		gboolean rs_ok;
+		rs_ok = server_resultset_check_data_model (rs, 4, 
+							   GDA_VALUE_TYPE_STRING, 
+							   GDA_VALUE_TYPE_STRING,
+							   GDA_VALUE_TYPE_STRING,
+							   GDA_VALUE_TYPE_STRING);
+		if (rs_ok) {
+			/* proceed loading the sequence */
+			total = gda_data_model_get_n_rows (rs);
+			now = 0;
+			while (now < total) {
+				value = gda_data_model_get_value_at (rs, 0, now);
 				str = gda_value_stringify (value);
-				list = find_user_name (db, str);
-				seq->owner = (gchar *) (list->data);
-				g_free (str);
-			}
-
-			if (created) {	/* then emit a signal */
+				created = FALSE;
+				seq = database_find_sequence_by_name (db, str);
+				if (!seq) {
+					seq = DB_SEQUENCE (db_sequence_new ());
+					seq->name = str;
+					str = NULL;
+					g_signal_connect (G_OBJECT (seq), "load_fault",
+							  G_CALLBACK (database_cb), db);
+					db->sequences = g_slist_append (db->sequences, seq);
+					created = TRUE;
+				}
+				if (str)
+					g_free (str);
+				
+				/* (re)computes the comments */
+				value = gda_data_model_get_value_at (rs, 2, now);
+				if (!gda_value_is_null (value)) {
+					str = gda_value_stringify (value);	/* str allocated */
+					if (seq->comments)
+						g_free (seq->comments);
+					seq->comments = str;
+				}
+				
+				/* (re)computes the owner */
+				value = gda_data_model_get_value_at (rs, 1, now);
+				if (!gda_value_is_null (value)) {
+					GSList *list;
+					str = gda_value_stringify (value);
+					list = find_user_name (db, str);
+					seq->owner = (gchar *) (list->data);
+					g_free (str);
+				}
+				
+				if (created) {	/* then emit a signal */
 #ifdef debug_signal
-				g_print (">> 'SEQ_CREATED' from database_load_sequences (%s)\n", seq->name);
+					g_print (">> 'SEQ_CREATED' from database_load_sequences (%s)\n", seq->name);
 #endif
-				g_signal_emit (G_OBJECT (db), database_signals[SEQ_CREATED], 0,
-					       seq);
+					g_signal_emit (G_OBJECT (db), database_signals[SEQ_CREATED], 0,
+						       seq);
 #ifdef debug_signal
-				g_print ("<< 'SEQ_CREATED' from database_load_sequences\n");
+					g_print ("<< 'SEQ_CREATED' from database_load_sequences\n");
 #endif
-			}
+				}
 
-			/* mark the sequence as updated */
-			DB_ITEM (seq)->updated = TRUE;
-			g_signal_emit_by_name (G_OBJECT (db), "progress",
-						 "SEQUENCES",
-						 now, total);
-			now++;
+				/* mark the sequence as updated */
+				DB_ITEM (seq)->updated = TRUE;
+				g_signal_emit_by_name (G_OBJECT (db), "progress",
+						       "SEQUENCES",
+						       now, total);
+				now++;
+			}
+			g_object_unref (G_OBJECT (rs));
 		}
-		g_object_unref (G_OBJECT (rs));
+		else
+			g_warning ("The provider has returned the wrong \n"
+				   "number and/or types of data\n"
+				   "for the SEQUENCE schema description.\n"
+				   "Please contact the provider's maintainer "
+				   "and report the problem.");
 
 		/* 2nd step: destroy sequences that do not exist anymore */
 		list = db->sequences;
@@ -1305,10 +1357,9 @@ database_delete_seq_field_link (Database * db, DbSequence * seq,
 			}
 		}
 		else
-			g_warning
-				("Database::database_delete_seq_field_link\nyou are "
-				 "trying to access "
-				 "the fields of the wrong Database structure!\n");
+			g_warning ("Database::database_delete_seq_field_link\nyou are "
+				   "trying to access "
+				   "the fields of the wrong Database structure!\n");
 	else /* field = NULL */ if (seq) {
 		while (seq->field_links) {
 			GSList *hold;
@@ -1933,37 +1984,55 @@ db_table_load_fields (DbTable * t, ServerAccess * srv, Database * db)
 	gda_parameter_list_free (paramlist);
 
 	if (rs) {
-		total = gda_data_model_get_n_rows (rs);
-		now = 0;		
-		while (now < total) {
-			value = gda_data_model_get_value_at (rs, 0, now);
-			str = gda_value_stringify (value);
-			field = db_table_find_field_by_name (t, str);
-			g_free (str);
-			if (field)
-				db_field_load_contents (srv, field, rs, now);
-			else {
-				field = DB_FIELD (db_field_new ());
-				g_signal_connect (G_OBJECT (field), "load_fault",
-						    G_CALLBACK (db_table_cb), t);
-				t->fields = g_slist_insert (t->fields, field, now);
-				/* updating the field's contents */
-				db_field_load_contents (srv, field, rs, now);
+		if (server_resultset_check_data_model (rs, 9, 
+						       GDA_VALUE_TYPE_STRING, 
+						       GDA_VALUE_TYPE_STRING, 
+						       GDA_VALUE_TYPE_INTEGER, 
+						       GDA_VALUE_TYPE_INTEGER,
+						       GDA_VALUE_TYPE_BOOLEAN,
+						       GDA_VALUE_TYPE_BOOLEAN,
+						       GDA_VALUE_TYPE_BOOLEAN,
+						       GDA_VALUE_TYPE_STRING,
+						       -1)) {
+			total = gda_data_model_get_n_rows (rs);
+			now = 0;		
+			while (now < total) {
+				value = gda_data_model_get_value_at (rs, 0, now);
+				str = gda_value_stringify (value);
+				field = db_table_find_field_by_name (t, str);
+				g_free (str);
+				if (field)
+					db_field_load_contents (srv, field, rs, now);
+				else {
+					field = DB_FIELD (db_field_new ());
+					g_signal_connect (G_OBJECT (field), "load_fault",
+							  G_CALLBACK (db_table_cb), t);
+					t->fields = g_slist_insert (t->fields, field, now);
+					/* updating the field's contents */
+					db_field_load_contents (srv, field, rs, now);
 #ifdef debug_signal
-				g_print (">> 'TAB_FIELD_CREATED' from db_table_load_fields\n");
+					g_print (">> 'TAB_FIELD_CREATED' from db_table_load_fields\n");
 #endif
-				g_signal_emit (G_OBJECT (t), db_table_signals[TAB_FIELD_CREATED], 0, field);
+					g_signal_emit (G_OBJECT (t), db_table_signals[TAB_FIELD_CREATED], 0, field);
 #ifdef debug_signal
-				g_print ("<< 'TAB_FIELD_CREATED' from db_table_load_fields\n");
+					g_print ("<< 'TAB_FIELD_CREATED' from db_table_load_fields\n");
 #endif
+				}
+				
+				/* mark the field as updated */
+				DB_ITEM (field)->updated = TRUE;
+				
+				now++;
 			}
-
-			/* mark the field as updated */
-			DB_ITEM (field)->updated = TRUE;
-
-			now++;
 		}
-		g_object_unref (G_OBJECT (rs));
+		else
+			g_warning ("The provider has returned the wrong \n"
+				   "number and/or types of data\n"
+				   "for the FIELDS schema description.\n"
+				   "Please contact the provider's maintainer "
+				   "and report the problem.");
+
+			g_object_unref (G_OBJECT (rs));
 
 		/* 2nd step: destroy fields that do not exist anymore */
 		db_table_clean_fields (db, t);
