@@ -144,6 +144,7 @@ object_selector_init (ObjectSelector * wid)
 	wid->priv->conf = NULL;
 	wid->priv->mode = 0;
 	wid->priv->treeview = NULL;
+	wid->priv->parent_selection = NULL;
 	wid->priv->selection = NULL;
 	wid->priv->objects_infos = NULL;
 	wid->priv->weak_ref_objects = NULL;
@@ -207,7 +208,8 @@ conn_to_close (ServerAccess *srv, ObjectSelector *os)
 			list = g_slist_next (list);
 		}
 		g_slist_free (os->priv->managers_infos);
-		
+		os->priv->managers_infos = NULL;
+
 		/* weak ref objects */
 		list = os->priv->weak_ref_objects;
 		while (list) {
@@ -221,6 +223,7 @@ conn_to_close (ServerAccess *srv, ObjectSelector *os)
 		g_warning ("ObjectSelector::conn_to_close() os->priv is NULL!\n");
 }
 
+static void tree_selection_changed_cb (GtkTreeSelection *select, ObjectSelector *os);
 static void
 object_selector_dispose (GObject   * object)
 {
@@ -232,11 +235,17 @@ object_selector_dispose (GObject   * object)
 
 	if (os->priv) {
 		GSList *list;
+		GtkTreeSelection *select;
+
 		/* signals, etc */
+		select = gtk_tree_view_get_selection (GTK_TREE_VIEW (os->priv->treeview));
+		g_signal_handlers_disconnect_by_func (G_OBJECT (select),
+						      G_CALLBACK (tree_selection_changed_cb), os);
 		conn_to_close (NULL, os);
 		
 		/* disconnect the "conn_to_close" signal */
-		g_signal_handlers_disconnect_by_func (G_OBJECT (os->priv->conf->srv), G_CALLBACK (conn_to_close), os);
+		g_signal_handlers_disconnect_by_func (G_OBJECT (os->priv->conf->srv), 
+						      G_CALLBACK (conn_to_close), os);
 		
 		/* logic */
 		list = os->priv->inner_logic;
@@ -267,7 +276,6 @@ object_selector_dispose (GObject   * object)
 
 
 
-static void tree_selection_changed_cb (GtkTreeSelection *select, ObjectSelector *os);
 static void tree_value_set_func (GtkTreeViewColumn *tree_column,
 				 GtkCellRenderer *cell,
 				 GtkTreeModel *model,
@@ -371,7 +379,6 @@ object_selector_initialize (ObjectSelector *os, va_list args)
 	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
 	g_signal_connect (G_OBJECT (select), "changed",
 			  G_CALLBACK (tree_selection_changed_cb), os);
-
 }
 
 static void
@@ -401,16 +408,30 @@ tree_selection_changed_cb (GtkTreeSelection *select, ObjectSelector *os)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
+	GObject *sel_obj = NULL;
 
 	if (gtk_tree_selection_get_selected (select, &model, &iter)) {
-		GObject *sel_obj;
-
 		gtk_tree_model_get (model, &iter, REF_COLUMN, &sel_obj, -1);
-		g_signal_emit (G_OBJECT (os), object_selector_signals[C_SELECTION_CHANGED], 0, sel_obj);
+		os->priv->selection = sel_obj;
+		os->priv->parent_selection = object_selector_get_selected_object_parent (os);
 	}
+	else {
+		os->priv->selection = NULL;
+		os->priv->parent_selection = NULL;
+	}
+
+#ifdef debug_signal
+	g_print (">> 'SELECTION_CHANGED' from tree_selection_changed_cb\n");
+#endif
+	g_signal_emit (G_OBJECT (os), object_selector_signals[C_SELECTION_CHANGED], 0, sel_obj);
+#ifdef debug_signal
+	g_print ("<< 'SELECTION_CHANGED' from tree_selection_changed_cb\n");
+#endif
+
 }
 
 
+
 /*
  * Returns a pointer to the OS's managed copy of the ObjectInfo equal to 'info'.
  * If it does not exist, then it is copied, added to the 'objects_infos' list and returned
@@ -531,9 +552,8 @@ weak_ref_object_not (ObjectSelector *os, GObject *obj)
 	
 	/* remove object from weak reference objects list */
 	list = g_slist_find (os->priv->weak_ref_objects, obj);
-	if (list) {
+	if (list) 
 		os->priv->weak_ref_objects = g_slist_delete_link (os->priv->weak_ref_objects, list);
-	}
 	else 
 		g_warning ("weak_ref_object_not(): can't find object");
 
@@ -745,7 +765,6 @@ void object_selector_add_manager (ObjectSelector * os,
 			list2 = g_slist_next (list2);
 		}
 		os->priv->managers_infos = g_slist_append (os->priv->managers_infos, minfo);
-		
 	}
 }
 
@@ -753,7 +772,7 @@ void object_selector_add_manager (ObjectSelector * os,
  * Add an object in the treeview. If the object will be a manager itself, then
  * the function object_selector_add_manager() MUST be called before this one.
  */
-void 
+ObjectInfo *
 object_selector_add_object (ObjectSelector * os, 
 			    GtkTreeIter *parent_iter, 
 			    GObject *parent,
@@ -764,17 +783,16 @@ object_selector_add_object (ObjectSelector * os,
 	GtkTreeIter iter, newcat;
 	TreeItem *item;
 	GtkTreeIter before_iter, *sibling, *cat;
-	const ObjectInfo *info = NULL;
+	ObjectInfo *info = NULL;
 	const ManagerInfo *minfo = NULL;
 	GSList *list;
 
-	g_return_if_fail (os && IS_OBJECT_SELECTOR (os));
+	g_return_val_if_fail (os && IS_OBJECT_SELECTOR (os), NULL);
 	if (parent)
-		g_return_if_fail (G_IS_OBJECT (parent));
-	g_return_if_fail (obj && G_IS_OBJECT (obj));
+		g_return_val_if_fail (G_IS_OBJECT (parent), NULL);
+	g_return_val_if_fail (obj && G_IS_OBJECT (obj), NULL);
 
 	model = gtk_tree_view_get_model (os->priv->treeview);
-
 	
 	/* find the right ObjectInfo */
 	if (!oinfo) {
@@ -806,7 +824,7 @@ object_selector_add_object (ObjectSelector * os,
 
 		if (!info) {
 			g_warning ("object_selector_add_object(): can't find information for object!");
-			return;
+			return NULL;
 		}
 	}
 	else
@@ -958,6 +976,8 @@ object_selector_add_object (ObjectSelector * os,
 	g_free (item->cmp_name);
 	g_free (item->comment);
 	g_free (item);
+
+	return info;
 }
 
 #ifdef debug
@@ -978,6 +998,7 @@ object_created_cb (GObject *obj, GObject *new_obj, ObjectSelector *os)
 {
 	GtkTreeModel *model;
 	GtkTreePath *path;
+	GtkTreeIter obj_iter;
 
 	/* if the object should be a manager, then set it as a manager */
 	object_selector_add_manager (os, new_obj, NULL);
@@ -988,13 +1009,21 @@ object_created_cb (GObject *obj, GObject *new_obj, ObjectSelector *os)
 
 	if (path) {
 		GtkTreeIter iter;
-		
-		if (gtk_tree_model_get_iter (model, &iter, path)) 
-			object_selector_add_object (os, &iter, obj, new_obj, NULL, NULL, TRUE);
+
+		if (gtk_tree_model_get_iter (model, &iter, path)) {
+			ObjectInfo *oinfo;
+			oinfo = object_selector_add_object (os, &iter, obj, new_obj, NULL, &obj_iter, TRUE);
+			if (oinfo->fill_children)  
+				(oinfo->fill_children) (new_obj, os, &obj_iter); /* add children objects */
+		}
 		gtk_tree_path_free (path);
 	}
-	else 
-		object_selector_add_object (os, NULL, obj, new_obj, NULL, NULL, TRUE);
+	else {
+		ObjectInfo *oinfo;
+		oinfo = object_selector_add_object (os, NULL, obj, new_obj, NULL, &obj_iter, TRUE);
+		if (oinfo->fill_children)  
+			(oinfo->fill_children) (new_obj, os, &obj_iter); /* add children objects */
+	}
 }
 
 static void
@@ -1038,20 +1067,50 @@ get_cut_path_depth (const gchar *path, guint depth, gboolean with_last)
 
 void
 object_selector_set_selection (ObjectSelector *os, GObject *selection)
+{
+	object_selector_set_selection_parent (os, selection, NULL);
+}
+
+void
+object_selector_set_selection_parent (ObjectSelector *os, GObject *selection, GObject *parent)
 {
 	GtkTreeModel *model;
-	GtkTreePath *path;
+	GtkTreePath *path, *parent_path = NULL;
+	GtkTreeIter iter, *parent_iter = NULL;
+	GtkTreeIter sub_iter;
 
 	g_return_if_fail (os && IS_OBJECT_SELECTOR (os));
 	g_return_if_fail (selection && G_IS_OBJECT (selection));
-	
+	if (parent)
+		g_return_if_fail (G_IS_OBJECT (parent));
+
 	model = gtk_tree_view_get_model (os->priv->treeview);
-	path = object_selector_find_model_path (model, selection, G_TYPE_INVALID);
+
+	if (parent) {
+		parent_path = object_selector_find_model_path (model, parent, G_TYPE_INVALID);
+		if (parent_path) {
+			if (gtk_tree_model_get_iter (model, &iter, parent_path)) 
+				parent_iter = &iter;
+			gtk_tree_path_free (parent_path);
+		}
+
+		if (parent_iter) {
+			/* going to the sub node */
+			if (gtk_tree_model_iter_children (model, &sub_iter, parent_iter))
+				parent_iter = &sub_iter;
+		}
+
+		if (!parent_iter)
+			g_warning ("object_selector_set_selection_parent(): can't find parent object!");
+
+	}
+		
+	path = find_model_path_real (model, selection, G_TYPE_INVALID, parent_iter);
 	if (path) {
 		gchar *strpath, *partpath;
 		guint i = 1;
 		GtkTreePath *ppath;
-		GtkTreeSelection *selection;
+		GtkTreeSelection *tselect;
 
 		strpath = gtk_tree_path_to_string (path);
 		partpath = get_cut_path_depth (strpath, i, FALSE);
@@ -1068,12 +1127,22 @@ object_selector_set_selection (ObjectSelector *os, GObject *selection)
 		g_free (strpath);
 		gtk_tree_view_scroll_to_cell (os->priv->treeview, path, NULL, TRUE,
 					      0.5, 0.);
-		selection = gtk_tree_view_get_selection (os->priv->treeview);
-		gtk_tree_selection_unselect_all (selection);
-		gtk_tree_selection_select_path (selection, path);
+		tselect = gtk_tree_view_get_selection (os->priv->treeview);
+
+		g_signal_handlers_block_by_func (G_OBJECT (tselect),
+						 G_CALLBACK (tree_selection_changed_cb), os);
+
+		gtk_tree_selection_unselect_all (tselect);
+		gtk_tree_selection_select_path (tselect, path);
+		os->priv->selection = selection;
+		os->priv->parent_selection = object_selector_get_selected_object_parent (os);
+
+		g_signal_handlers_unblock_by_func (G_OBJECT (tselect),
+						   G_CALLBACK (tree_selection_changed_cb), os);
+
 		gtk_tree_view_set_cursor (os->priv->treeview, path, NULL, FALSE);
 		gtk_tree_path_free (path);
-	}
+	}	
 }
 
 void
@@ -1106,3 +1175,38 @@ object_selector_set_column_label (ObjectSelector *os, guint column, const gchar
 	tcolumn = gtk_tree_view_get_column (os->priv->treeview, column);
 	gtk_tree_view_column_set_title (tcolumn, label);
 }
+
+GObject *
+object_selector_get_selected_object (ObjectSelector *os)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	GtkTreeSelection *select;
+	GObject *sel_obj = NULL;
+
+	select = gtk_tree_view_get_selection (os->priv->treeview);
+	
+	if (gtk_tree_selection_get_selected (select, &model, &iter)) 
+		gtk_tree_model_get (model, &iter, REF_COLUMN, &sel_obj, -1);
+	
+	return sel_obj;
+}
+
+GObject *
+object_selector_get_selected_object_parent (ObjectSelector *os)
+{
+	GtkTreeIter iter, parent_iter;
+	GtkTreeModel *model;
+	GtkTreeSelection *select;
+	GObject *sel_obj = NULL;
+
+	select = gtk_tree_view_get_selection (os->priv->treeview);
+	
+	if (gtk_tree_selection_get_selected (select, &model, &iter)) {
+		if (gtk_tree_model_iter_parent (model, &parent_iter, &iter)) {
+			gtk_tree_model_get (model, &parent_iter, REF_COLUMN, &sel_obj, -1);
+		}
+	}
+	
+	return sel_obj;
+}
