@@ -170,6 +170,7 @@ query_editor_expr_new (QueryEditorFields * qef, QueryField *qf)
 	qee->orig_qf = qf;
 
 	query_editor_expr_initialize (qee);
+
 	if (qf) {
 		/* make a copy of qf to be the QueryField we are working on 
 		   => we now have a default reference on ALL the objects in the fields list */
@@ -581,8 +582,9 @@ qee_refresh (QueryEditorExpr * qee)
 		gtk_widget_destroy (qee->priv->current_area);
 		qee->priv->current_area = NULL;
 	}
-	if (qee->priv->sel_qf)
+	if (qee->priv->sel_qf && qee->top_qf) {
 		area = query_field_get_edit_widget (qee->priv->sel_qf);
+	}
 	else {
 		area = gtk_label_new ("");
 		gtk_widget_set_size_request (area, 200, 100);
@@ -675,46 +677,60 @@ expr_qf_changed_cb (QueryField *qf, QueryEditorExpr *qee)
 
 	/* simple rules to set default names and print names */
 	if (qee->priv->sel_qf) {
-		gchar *str = NULL;
+		const gchar *cstr = NULL;
 		QueryFieldType ftype = query_field_get_ftype (qee->priv->sel_qf);
 		QueryView *view;
+		ServerFunction *function;
 
 		switch (ftype) {
 		case QUERY_FIELD_FIELD:
 			if (query_field_field_is_db_field (qee->priv->sel_qf)) {
 				DbField *field = query_field_field_get_db_field (qee->priv->sel_qf);
 				if (field)
-					str = field->name;
+					cstr = field->name;
 			}
 			else {
 				QueryField *field = query_field_field_get_query_field (qee->priv->sel_qf);
 				if (field)
-					str = query_field_get_name (field);
+					cstr = query_field_get_name (field);
 			}
-			if (str) {
-				gtk_entry_set_text (GTK_ENTRY (qee->priv->name_entry), str);
+			if (cstr) {
+				gtk_entry_set_text (GTK_ENTRY (qee->priv->name_entry), cstr);
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (qee->priv->printed_cb), TRUE);
-				gtk_entry_set_text (GTK_ENTRY (qee->priv->print_name_entry), str);
+				gtk_entry_set_text (GTK_ENTRY (qee->priv->print_name_entry), cstr);
 			}
 			break;
 		case QUERY_FIELD_ALLFIELDS:
 			view = query_field_allfields_get_queryview (qee->priv->sel_qf);
 			if (view) {
-				str = query_view_get_name (view);
+				cstr = query_view_get_name (view);
 				
-				gtk_entry_set_text (GTK_ENTRY (qee->priv->name_entry), str);
+				gtk_entry_set_text (GTK_ENTRY (qee->priv->name_entry), cstr);
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (qee->priv->printed_cb), TRUE);
 				gtk_entry_set_text (GTK_ENTRY (qee->priv->print_name_entry), "");
 			}
 			break;
 		case QUERY_FIELD_VALUE:
-			if (query_field_value_is_parameter (qee->priv->sel_qf))
-				str = _("Parameter");
-			else
-				str = _("Value");
-			gtk_entry_set_text (GTK_ENTRY (qee->priv->name_entry), str);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (qee->priv->printed_cb), FALSE);
-			gtk_entry_set_text (GTK_ENTRY (qee->priv->print_name_entry), "");
+			cstr = query_field_get_name (qee->priv->sel_qf);
+			if (!cstr || (cstr && !*cstr)) {
+				if (query_field_value_is_parameter (qee->priv->sel_qf))
+					cstr = _("Parameter");
+				else
+					cstr = _("Value");
+				gtk_entry_set_text (GTK_ENTRY (qee->priv->name_entry), cstr);
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (qee->priv->printed_cb), FALSE);
+				gtk_entry_set_text (GTK_ENTRY (qee->priv->print_name_entry), "");
+			}
+			break;
+		case QUERY_FIELD_FUNCTION:
+			function = query_field_func_get_function (qee->priv->sel_qf);
+			if (function) {
+				cstr = function->sqlname;
+				
+				gtk_entry_set_text (GTK_ENTRY (qee->priv->name_entry), cstr);
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (qee->priv->printed_cb), TRUE);
+				gtk_entry_set_text (GTK_ENTRY (qee->priv->print_name_entry), cstr);
+			}
 			break;
 		default:
 			break;
@@ -829,6 +845,7 @@ query_editor_expr_fetch_fields (QueryEditorExpr * qee)
 	AAA (1000);
 	g_print ("query_editor_expr_fetch_fields() fields are:\n");
 	dump_fields (NULL, qee);
+	AAA (1001);
 #endif
 
 	/* disconnects the "field_modified" signal from the QueryFields */
@@ -920,6 +937,17 @@ dump_fields (GtkWidget *button, QueryEditorExpr * qee)
 		g_print ("%s\tid: %d\n", str, query_field_get_id (qf));
 		g_print ("%s\talias : %s\n", str, query_field_get_alias (qf));
 		g_print ("%s\tfield type: %s\n", str, field_type_to_str (query_field_get_ftype (qf)));
+		if (query_field_get_ftype (qf) == QUERY_FIELD_VALUE) {
+			const GdaValue *val;
+			g_print ("%s\tVALUE: ServerDataType GDA value = %d\n", str,
+				 query_field_value_get_data_type (qf)->gda_type);
+			val = query_field_value_get_value (qf);
+			if (val)
+				g_print ("%s\tVALUE: GdaValue = %p, type = %d\n", str, val,
+					 gda_value_get_type (val));
+			else
+				g_print ("%s\tVALUE: GdaValue = %p\n", str, val);
+		}
 		monitored = query_field_get_monitored_objects (qf);
 		iter = monitored;
 		while (iter) {
