@@ -19,7 +19,7 @@
  */
 
 #include <config.h>
-#include <libgnomedb/gnome-db-sql-editor.h>
+#include <libgnomedb/gnome-db-editor.h>
 #include "query-editor.h"
 #include "query-editor-fields.h"
 #include "relship.h"
@@ -212,7 +212,7 @@ query_changed_cb (Query *q, QueryEditor *qed)
 	gchar *str;
 
 	str = query_get_select_query (q, NULL);
-	gnome_db_sql_editor_set_text (GNOME_DB_SQL_EDITOR (qed->priv->info_sql), str, -1);
+	gnome_db_editor_set_text (GNOME_DB_EDITOR (qed->priv->info_sql), str, -1);
 	g_free (str);
 }
 
@@ -325,7 +325,7 @@ query_editor_initialize (QueryEditor * qed)
 	sw = gtk_frame_new (NULL);
 	gtk_frame_set_shadow_type (GTK_FRAME (sw), GTK_SHADOW_IN);
 	gtk_box_pack_start (GTK_BOX (vb), sw, TRUE, TRUE, GNOME_PAD/2.);
-	qed->priv->sql_view = gnome_db_sql_editor_new ();
+	qed->priv->sql_view = gnome_db_editor_new ();
 	gtk_container_add (GTK_CONTAINER (sw), qed->priv->sql_view);
 
 	bb = gtk_hbutton_box_new ();
@@ -363,8 +363,8 @@ query_editor_initialize (QueryEditor * qed)
 	gtk_container_set_border_width (GTK_CONTAINER (frame), GNOME_PAD/2.);
 	gtk_paned_pack1 (GTK_PANED (vp2), frame, TRUE, TRUE);
 	
-	wid = gnome_db_sql_editor_new ();
-	gnome_db_sql_editor_set_editable (GNOME_DB_SQL_EDITOR (wid), FALSE);
+	wid = gnome_db_editor_new ();
+	gnome_db_editor_set_editable (GNOME_DB_EDITOR (wid), FALSE);
 	gtk_container_add (GTK_CONTAINER (frame), wid);
 	qed->priv->info_sql = wid;
 
@@ -395,7 +395,8 @@ query_editor_initialize (QueryEditor * qed)
 	gtk_container_set_border_width (GTK_CONTAINER (frame), GNOME_PAD/2.);
 	gtk_paned_add2 (GTK_PANED (hp), frame);
 
-	wid = gtk_label_new ("Nothing Yet!");
+	wid = conds_view_new_in_sw (qed->priv->query);
+	gtk_widget_set_size_request (wid, 250, 250);
 	gtk_container_add (GTK_CONTAINER (frame), wid);
 
 	
