@@ -195,16 +195,16 @@ prepare_query (QueryExec * qx)
 	list = qx->priv->q->fields;
 	while (list) {
 		qf = QUERY_FIELD (list->data);
-		if (qf->field_type == QUERY_FIELD_ALLFIELDS) {
+		if (query_field_get_ftype (qf) == QUERY_FIELD_ALLFIELDS) {
 			GSList *nextone = g_slist_next (list);
 			QueryView *view = query_field_allfields_get_queryview (qf);
 
-			if (view && IS_DB_TABLE (view->obj)) {
+			if (view && IS_DB_TABLE (query_view_get_view_obj (view))) {
 				DbTable *table;
 				QueryField *nqf;
 				GSList *fields;
 
-				table = DB_TABLE (view->obj);
+				table = DB_TABLE (query_view_get_view_obj (view));
 				fields = table->fields;
 				while (fields) {
 					nqf = query_field_field_dbfield_new (qx->priv->q,
@@ -228,8 +228,8 @@ prepare_query (QueryExec * qx)
 	while (list) {
 		QueryView *view = QUERY_VIEW (list->data);
 
-		if (IS_DB_TABLE (view->obj)) {
-			DbTable *table = DB_TABLE (view->obj);
+		if (IS_DB_TABLE (query_view_get_view_obj (view))) {
+			DbTable *table = DB_TABLE (query_view_get_view_obj (view));
 			gboolean all_pk_found = TRUE;
 			gboolean has_a_field = FALSE;
 			GSList *fields;
@@ -483,7 +483,7 @@ query_exec_run (QueryExec * qx)
 		mdn->qf = qf;
 		g_object_weak_ref (G_OBJECT (qf), (GWeakNotify) query_field_notify, qx);
 		
-		mdn->descr = g_strdup_printf ("%s:", qf->name);
+		mdn->descr = g_strdup_printf ("%s:", query_field_get_name (qf));
 		
 		/* FIXME: in the next function, use the right 2nd argument in relation of the implementation
 		   of the QUERY_FIELD_VALUE QueryField  */
@@ -491,7 +491,7 @@ query_exec_run (QueryExec * qx)
 		/* FIXME: use the default value defined by the QUERY_FIELD_VALUE QueryField 
 		   instead of ""*/
 		mdn->data_entry = GTK_WIDGET (data_handler_get_widget_from_value (dh,
-						       (query_field_value_get_default_value (qf))));
+						   (query_field_value_get_value (qf))));
 		missing_list = g_slist_append (missing_list, mdn);
 		list = g_slist_next (list);
 	}
