@@ -28,13 +28,15 @@
 #include "../query.h"
 #include "../query-field-private.h"
 #include "../database.h"
+#include "../object-selector.h"
 
 static void        q_init            (QueryField *qf);
 static void        q_finalize         (QueryField *qf);
 static void        q_deactivate      (QueryField *qf);
 static void        q_activate        (QueryField *qf);
 static GtkWidget * q_get_edit_widget (QueryField *qf);
-static GtkWidget * q_get_sel_widget  (QueryField *qf, GCallback callback, gpointer data);
+static GtkWidget * q_get_sel_widget  (QueryField *qf, QueryField *sel_qf, 
+				      GCallback callback, gpointer data);
 static gchar     * q_render_as_sql   (QueryField *qf, GSList * missing_values);
 static xmlNodePtr  q_render_as_xml   (QueryField *qf, GSList * missing_values);
 static gchar     * q_render_as_string(QueryField *qf, GSList * missing_values);
@@ -47,7 +49,6 @@ static void        q_replace_comp    (QueryField *qf, gint ref, GObject   *old,
 
 /* Weak ref from the refering object */
 static void        q_obj_destroyed_cb (QueryField *qf, GObject   *obj);
-static void        arg_destroyed_cb (QueryField *qf, QueryField *arg); 
 
 typedef struct {
 	ServerFunction *func;
@@ -55,8 +56,6 @@ typedef struct {
 	/* XML field name if we did not find it in the first place (like TVxxx:FIyyy) */
 	gchar          *func_name;
 
-	GSList         *weak_ref_objects;
-
 	/* list of references to arguments (struct Arg)  */
 	GSList         *args;
 } private_data;
@@ -69,7 +68,7 @@ typedef struct {
 } Arg;
 
 
-#define QF_PRIVATE_DATA(qf) ((private_data *) qf->private_data)
+#define QF_PRIVATE_DATA(qf) ((private_data *) qf->priv->private_data)
 #define QF_FUNC_ARG(arg) ((Arg *) (arg))
 
 QueryFieldIface * 
@@ -121,30 +120,33 @@ q_init            (QueryField *qf)
 {
 	private_data *data;
 	data = g_new0 (private_data, 1);
-	qf->private_data = (gpointer) data;
+	qf->priv->private_data = (gpointer) data;
 	QF_PRIVATE_DATA(qf)->func = NULL;
 	QF_PRIVATE_DATA(qf)->func_name = NULL;
 	QF_PRIVATE_DATA(qf)->args = NULL;
-	QF_PRIVATE_DATA(qf)->weak_ref_objects = NULL;
 }
 
+static void qf_set_func_ptr (QueryField *qf, ServerFunction *func);
 static void free_args (QueryField *qf);
 
 static void        
 q_finalize         (QueryField *qf)
 {
 	q_deactivate (qf);
-	if (qf->private_data) {
+	qf_set_func_ptr (qf, NULL);
+
+	if (qf->priv->private_data) {
 		if (QF_PRIVATE_DATA(qf)->func_name)
 			g_free (QF_PRIVATE_DATA(qf)->func_name);
 
 		free_args (qf);
 
-		g_free (qf->private_data);
-		qf->private_data = NULL;
+		g_free (qf->priv->private_data);
+		qf->priv->private_data = NULL;
 	}
 }
 
+static void release_arg (QueryField *qf, Arg *arg, gboolean retreive_xml);
 static void 
 free_args (QueryField *qf)
 {
@@ -152,40 +154,30 @@ free_args (QueryField *qf)
 
 	list = QF_PRIVATE_DATA(qf)->args;
 	while (list) {
-		if (QF_FUNC_ARG(list->data)->xml)
-			g_free (QF_FUNC_ARG(list->data)->xml);
-		
-		if (QF_FUNC_ARG(list->data)->qf &&
-		    g_slist_find (QF_PRIVATE_DATA(qf)->weak_ref_objects, QF_FUNC_ARG(list->data)->qf)) {
-			g_object_weak_unref (G_OBJECT (QF_FUNC_ARG(list->data)->qf), 
-					     (GWeakNotify) arg_destroyed_cb, qf);
-			QF_PRIVATE_DATA(qf)->weak_ref_objects = g_slist_remove (QF_PRIVATE_DATA(qf)->weak_ref_objects,
-										QF_FUNC_ARG(list->data)->qf);	
-		}
+		release_arg (qf, QF_FUNC_ARG (list->data), FALSE);
 		g_free (list->data);
 		
 		list = g_slist_next (list);
 	}
 	g_slist_free (QF_PRIVATE_DATA(qf)->args);
+
 	QF_PRIVATE_DATA(qf)->args = NULL;
 }
 
-static void qf_set_func_ptr (QueryField *qf, ServerFunction *func);
 static void sub_qf_status_changed_cb (QueryField *qf, QueryField *pqf);
+static void sub_qf_nullified_cb (QueryField *qf, QueryField *pqf);
+
 static void
 q_deactivate      (QueryField *qf)
 {
 	GSList *list;
-	g_print ("q_deactivate (%p)\n", qf);
 
 	/* This function disconnects any event handler from any object
 	   this QueryField wants to receive events from.
 	   Here we disconnect from the function if we are connected */
 
-	if (!qf->activated) {
-		g_print ("Not activated, returning!\n");
+	if (!qf->priv->activated) 
 		return;
-	}
 
 	/* Function part */
 	if (QF_PRIVATE_DATA(qf)->func) {
@@ -203,29 +195,7 @@ q_deactivate      (QueryField *qf)
 	list = QF_PRIVATE_DATA(qf)->args;
 	while (list) {
 		/* unreference that QueryField argument */
-		/* g_print ("Now calls query_field_free_ref (%p)\n", QF_FUNC_ARG(list->data)->qf); */
-/* 		query_field_free_ref (QF_FUNC_ARG(list->data)->qf); */		
-		
-		if (QF_FUNC_ARG(list->data)->xml &&
-		    QF_FUNC_ARG(list->data)->qf) {
-			g_free (QF_FUNC_ARG(list->data)->xml);
-			QF_FUNC_ARG(list->data)->xml = NULL;
-		}
-
-		if (QF_FUNC_ARG(list->data)->qf) {
-			if (g_slist_find (QF_PRIVATE_DATA(qf)->weak_ref_objects, QF_FUNC_ARG(list->data)->qf)) {
-				g_object_weak_unref (G_OBJECT (QF_FUNC_ARG(list->data)->qf), 
-						     (GWeakNotify) arg_destroyed_cb, qf);
-				QF_PRIVATE_DATA(qf)->weak_ref_objects = 
-					g_slist_remove (QF_PRIVATE_DATA(qf)->weak_ref_objects,
-							QF_FUNC_ARG(list->data)->qf);
-			}
-
-			QF_FUNC_ARG(list->data)->xml = query_field_get_xml_id (QF_FUNC_ARG(list->data)->qf);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (QF_FUNC_ARG(list->data)->qf),
-							      G_CALLBACK (sub_qf_status_changed_cb), qf);
-		}
-		QF_FUNC_ARG(list->data)->qf = NULL;
+		release_arg (qf, QF_FUNC_ARG(list->data), TRUE);
 
 		list = g_slist_next (list);
 	}
@@ -233,6 +203,28 @@ q_deactivate      (QueryField *qf)
 	query_field_set_activated (qf, FALSE);
 }
 
+static void 
+release_arg (QueryField *qf, Arg *arg, gboolean retreive_xml)
+{
+	if (arg->xml && arg->qf) {
+		g_free (arg->xml);
+		arg->xml = NULL;
+	}
+	
+	if (arg->qf) {
+		if (retreive_xml)
+			arg->xml = query_field_get_xml_id (arg->qf);
+
+		/* disconnecting signals */
+		g_signal_handlers_disconnect_by_func (G_OBJECT (arg->qf),
+						      G_CALLBACK (sub_qf_nullified_cb), qf);
+		g_signal_handlers_disconnect_by_func (G_OBJECT (arg->qf),
+						      G_CALLBACK (sub_qf_status_changed_cb), qf);
+	}
+	arg->qf = NULL;	
+}
+
+
 static QueryField *get_field_from_xml (GSList *list, gchar *xmlid);
 static void
 q_activate        (QueryField *qf)
@@ -241,17 +233,13 @@ q_activate        (QueryField *qf)
 	   receive events from. */
 	ServerFunction *func;
 
-	g_print ("q_activate (%p)\n", qf);
-	
-	if (qf->activated) {
-		g_print ("Already activated, returning!\n");
+	if (qf->priv->activated) 
 		return;
-	}
 
 	func = QF_PRIVATE_DATA(qf)->func;
 	
 	if (!func && QF_PRIVATE_DATA(qf)->func_name) {
-		func = server_function_get_from_xml_id (qf->query->conf->srv, 
+		func = server_function_get_from_xml_id (qf->priv->query->conf->srv, 
 							QF_PRIVATE_DATA(qf)->func_name);
 		if (func) {
 			g_free (QF_PRIVATE_DATA(qf)->func_name);
@@ -277,28 +265,28 @@ q_activate        (QueryField *qf)
 					
 					/* fetch the list of QueryFields to work with */
 					qfs = g_object_get_data (G_OBJECT (qf), "qf_list");
-
+					
 					if (!qfs)
-						qfs = &(qf->query->fields);
+						qfs = &(qf->priv->query->fields);
 					
 					/* fetch the argument QueryField */
 					aqf = get_field_from_xml (*qfs, QF_FUNC_ARG(list->data)->xml);
 
 					if (aqf) {
 						QF_FUNC_ARG(list->data)->qf = aqf;
-						g_object_weak_ref (G_OBJECT (aqf), 
-								   (GWeakNotify) arg_destroyed_cb, qf);
-						QF_PRIVATE_DATA(qf)->weak_ref_objects = 
-							g_slist_append (QF_PRIVATE_DATA(qf)->weak_ref_objects,
-									aqf);
 
-						if (! aqf->activated) {
+						if (! aqf->priv->activated) {
 							query_field_activate (aqf);
-							if (! aqf->activated)
+							if (! aqf->priv->activated)
 								allfound = FALSE;
 						}
+
+						g_signal_connect (G_OBJECT (aqf), "nullified",
+								  G_CALLBACK (sub_qf_nullified_cb), qf);
 						g_signal_connect (G_OBJECT (aqf), "status_changed",
 								  G_CALLBACK (sub_qf_status_changed_cb), qf);
+
+						g_object_ref (G_OBJECT (aqf));
 					}
 					else 
 						allfound = FALSE;
@@ -307,9 +295,9 @@ q_activate        (QueryField *qf)
 					allfound = FALSE;
 			}
 			else {
-				if (! QF_FUNC_ARG(list->data)->qf->activated) {
+				if (! QF_FUNC_ARG(list->data)->qf->priv->activated) {
 					query_field_activate (QF_FUNC_ARG(list->data)->qf);
-					if (! QF_FUNC_ARG(list->data)->qf->activated)
+					if (! QF_FUNC_ARG(list->data)->qf->priv->activated)
 						allfound = FALSE;
 				}
 			}
@@ -326,19 +314,8 @@ q_activate        (QueryField *qf)
 		}
 
 		/* pronounce the activation if OK */
-		if (allfound) {
-			GSList *list;
-
-			/* add a reference to the QueryField object as arguments */
-			list = QF_PRIVATE_DATA(qf)->args;
-			while (list) {
-				query_field_use_ref (QF_FUNC_ARG(list->data)->qf);
-				list = g_slist_next (list);
-			}
-
-			/* now it's activated */
+		if (allfound) 
 			query_field_set_activated (qf, TRUE);
-		}
 	}
 }
 
@@ -357,37 +334,66 @@ qf_set_func_ptr (QueryField *qf, ServerFunction *func)
 		return;
 
 	if (QF_PRIVATE_DATA(qf)->func) {
-		if (g_slist_find (QF_PRIVATE_DATA(qf)->weak_ref_objects, QF_PRIVATE_DATA(qf)->func)) {
-			g_object_weak_unref (G_OBJECT (QF_PRIVATE_DATA(qf)->func), 
-					     (GWeakNotify) q_obj_destroyed_cb, qf);
-			QF_PRIVATE_DATA(qf)->weak_ref_objects = g_slist_remove (QF_PRIVATE_DATA(qf)->weak_ref_objects,
-										QF_PRIVATE_DATA(qf)->func);
-		}
-
+		g_object_weak_unref (G_OBJECT (QF_PRIVATE_DATA(qf)->func), 
+				     (GWeakNotify) q_obj_destroyed_cb, qf);
+		
 		QF_PRIVATE_DATA(qf)->func = NULL;
 	}
 
 	if (func) {
 		QF_PRIVATE_DATA(qf)->func = func;
 		
-		if (!g_slist_find (QF_PRIVATE_DATA(qf)->weak_ref_objects, QF_PRIVATE_DATA(qf)->func)) {
-			g_object_weak_ref (G_OBJECT (QF_PRIVATE_DATA(qf)->func), 
-					   (GWeakNotify) q_obj_destroyed_cb, qf);
-			QF_PRIVATE_DATA(qf)->weak_ref_objects = g_slist_append (QF_PRIVATE_DATA(qf)->weak_ref_objects,
-										QF_PRIVATE_DATA(qf)->func);	
-		}
+		g_object_weak_ref (G_OBJECT (QF_PRIVATE_DATA(qf)->func), 
+				   (GWeakNotify) q_obj_destroyed_cb, qf);
 	}
 }
 
 static void 
 sub_qf_status_changed_cb (QueryField *qf, QueryField *pqf)
 {
-	if (qf->activated)
+	if (qf->priv->activated)
 		query_field_activate (pqf);
 	else 
 		query_field_set_activated (pqf, FALSE);
 }
 
+static void
+sub_qf_nullified_cb (QueryField *qf, QueryField *pqf)
+{
+	/* the steps are to deactivate the function, and empyt the arg */
+	GSList *list;
+	Arg *found = NULL;
+	
+	list = QF_PRIVATE_DATA(pqf)->args;
+	while (list && !found) {
+		if (QF_FUNC_ARG(list->data)->qf == qf)
+			found = QF_FUNC_ARG(list->data);
+		list = g_slist_next (list);
+	}
+
+	if (!found) {
+		g_warning ("Can't find argument destroyed!");
+		return;
+	}
+
+	/* disconnect signals */
+	release_arg (pqf, found, FALSE);
+
+	/* lose the reference on that qf */
+	g_object_unref (G_OBJECT (qf));
+
+	query_field_deactivate (pqf);
+
+#ifdef debug_signal
+	g_print (">> 'FIELD_MODIFIED' from query-fields/sub_qf_nullified_cb\n");
+#endif
+	g_signal_emit_by_name (G_OBJECT (pqf), "field_modified");
+#ifdef debug_signal
+	g_print ("<< 'FIELD_MODIFIED' from query-fields/sub_qf_nullified_cb\n");
+#endif	
+}
+
+
 static QueryField *
 get_field_from_xml (GSList *list, gchar *xmlid)
 {
@@ -409,7 +415,7 @@ get_field_from_xml (GSList *list, gchar *xmlid)
 
 	token = list;
 	while (token && !qf) {
-		if (QUERY_FIELD (token->data)->id == id)
+		if (QUERY_FIELD (token->data)->priv->id == id)
 			qf = QUERY_FIELD (token->data);
 		token = g_slist_next (token);
 	}
@@ -419,214 +425,79 @@ get_field_from_xml (GSList *list, gchar *xmlid)
 
 
 static void        
-q_obj_destroyed_cb (QueryField *qf, GObject   *obj)
+q_obj_destroyed_cb (QueryField *qf, GObject *obj)
 {
-	QF_PRIVATE_DATA(qf)->weak_ref_objects = g_slist_remove (QF_PRIVATE_DATA(qf)->weak_ref_objects,
-								QF_PRIVATE_DATA(qf)->func);	
+	g_assert (obj == G_OBJECT (QF_PRIVATE_DATA(qf)->func));
+
+	QF_PRIVATE_DATA(qf)->func = NULL;
 	q_deactivate (qf);
 	
-	/* if the field or table disappear, then destroy is the result */
-	g_object_unref (G_OBJECT (qf));
-}
-
-static void
-arg_destroyed_cb (QueryField *qf, QueryField *arg)
-{
-	/* the steps are to deactivate the function, and empyt the arg */
-	GSList *list;
-	Arg *found = NULL;
-	
-	list = QF_PRIVATE_DATA(qf)->args;
-	while (list && !found) {
-		if (QF_FUNC_ARG(list->data)->qf == arg)
-			found = QF_FUNC_ARG(list->data);
-		list = g_slist_next (list);
-	}
-
-	if (!found) {
-		g_warning ("Can't find argument destroyed!");
-		return;
-	}
-
-	QF_PRIVATE_DATA(qf)->weak_ref_objects = g_slist_remove (QF_PRIVATE_DATA(qf)->weak_ref_objects,
-								arg);
-	query_field_deactivate (qf);
-	found->qf = NULL;
-	if (found->xml) { 
-		g_free (found->xml);
-		found->xml = NULL;
-	}
-
-#ifdef debug_signal
-	g_print (">> 'FIELD_MODIFIED' from query-fields/arg_destroyed_cb\n");
-#endif
-	g_signal_emit_by_name (G_OBJECT (qf), "field_modified");
-#ifdef debug_signal
-	g_print ("<< 'FIELD_MODIFIED' from query-fields/arg_destroyed_cb\n");
-#endif	
+	/* if the function disappear, then this QueryField must be nullified */
+	query_field_nullify (qf);
 }
 
 
-static void user_defined_functions_toggled_cb (GtkWidget * wid, GtkCList *clist);
-static void system_defined_functions_toggled_cb (GtkWidget * wid, GtkCList *clist);
-static void function_selected_cb (GtkWidget * wid, gint row, gint column,
-				  GdkEventButton * event, GtkCList *clist);
+
+static void function_selected_cb (ObjectSelector *os, GObject *sel, QueryField *qf);
+
 static GtkWidget * 
 q_get_edit_widget (QueryField *qf)
 {
-	GtkWidget *frame, *vb, *hb, *sw, *clist, *user, *sys;
-	gchar *titles[] = { N_("Function"), N_("Description") };
+	GtkWidget *vbox, *os;
 
-	frame = gtk_frame_new (_("Function"));
-	gtk_container_set_border_width (GTK_CONTAINER (frame), GNOME_PAD/2.);
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
 
-	vb = gtk_vbox_new (FALSE, GNOME_PAD/2.);
-	gtk_container_add (GTK_CONTAINER (frame), vb);
-	gtk_container_set_border_width (GTK_CONTAINER (vb), GNOME_PAD/2.);
+	os = object_selector_new (qf->priv->query->conf, OBJECT_SELECTOR_PROCS, -1);
+	object_selector_set_column_label (OBJECT_SELECTOR (os), 0, _("Function"));
+	if (QF_PRIVATE_DATA(qf)->func) 
+		object_selector_set_selection (OBJECT_SELECTOR (os), G_OBJECT (QF_PRIVATE_DATA(qf)->func));
+	gtk_box_pack_start (GTK_BOX (vbox), os, TRUE, TRUE, 0);
+	gtk_widget_show (os);
 
-	hb = gtk_hbox_new (TRUE, GNOME_PAD);
-	gtk_box_pack_start (GTK_BOX (vb), hb, FALSE, TRUE, 0);
+	g_signal_connect (G_OBJECT (os), "selection_changed",
+			  G_CALLBACK (function_selected_cb), qf);
 
-	clist = gtk_clist_new_with_titles (2, titles);
-	g_object_set_data (G_OBJECT (clist), "qf", qf);
-
-	user = gtk_check_button_new_with_label (_("User defined functions"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (user), TRUE);
-
-	g_signal_connect (G_OBJECT (user), "toggled",
-			    G_CALLBACK (user_defined_functions_toggled_cb), clist);
-	gtk_box_pack_start (GTK_BOX (hb), user, FALSE, TRUE, GNOME_PAD / 2);
-
-	sys = gtk_check_button_new_with_label (_("System functions"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sys), FALSE);
-	g_signal_connect (G_OBJECT (sys), "toggled",
-			    G_CALLBACK (system_defined_functions_toggled_cb), clist);
-	gtk_box_pack_start (GTK_BOX (hb), sys, FALSE, TRUE, GNOME_PAD / 2);
-
-	sw = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-					GTK_POLICY_AUTOMATIC,
-					GTK_POLICY_ALWAYS);
-	gtk_box_pack_start (GTK_BOX (vb), sw, TRUE, TRUE, 0);
-
-	gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
-	gtk_clist_set_reorderable (GTK_CLIST (clist), FALSE);
-	gtk_container_add (GTK_CONTAINER (sw), clist);
-	gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 0, TRUE);
-	gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 1, TRUE);
-	gtk_clist_column_titles_passive (GTK_CLIST (clist));
-	
-	gtk_widget_show_all (vb);
-
-	if (QF_PRIVATE_DATA(qf)->func) {
-		gint row;
-		if (QF_PRIVATE_DATA(qf)->func->is_user)
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (user), TRUE);
-		else
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sys), TRUE);
-		
-		row = gtk_clist_find_row_from_data (GTK_CLIST (clist), QF_PRIVATE_DATA(qf)->func);
-		gtk_clist_select_row (GTK_CLIST (clist), row, 0);
-		gtk_clist_moveto (GTK_CLIST (clist), row, 0, 0, 0);
-	}
-
-	g_signal_connect (G_OBJECT (clist), "select_row",
-			    G_CALLBACK (function_selected_cb), clist);
-
-	return frame;
-}
-
-static void populate_functions (GtkCList *clist);
-
-static void 
-user_defined_functions_toggled_cb (GtkWidget * wid, GtkCList *clist)
-{
-	g_object_set_data (G_OBJECT (clist), "user_func", 
-			     GINT_TO_POINTER (GTK_TOGGLE_BUTTON (wid)->active));
-	gtk_clist_freeze (clist);
-	gtk_clist_clear (clist);
-	populate_functions (clist);
-	gtk_clist_thaw (clist);
+	return vbox;
 }
 
 static void 
-system_defined_functions_toggled_cb (GtkWidget * wid, GtkCList *clist)
+function_selected_cb (ObjectSelector *os, GObject *sel, QueryField *qf)
 {
-	g_object_set_data (G_OBJECT (clist), "sys_func", 
-			     GINT_TO_POINTER (GTK_TOGGLE_BUTTON (wid)->active));
-	gtk_clist_freeze (clist);
-	gtk_clist_clear (clist);
-	populate_functions (clist);
-	gtk_clist_thaw (clist);
-}
+	if (IS_SERVER_FUNCTION (sel)) {
+		ServerFunction *func;
+		func = SERVER_FUNCTION (sel);
 
-static void 
-populate_functions (GtkCList *clist)
-{
-	gchar *line[2];
-	gint i;
-	gboolean user, sys;
-	QueryField *qf;
-	GSList *list;
-
-	qf = QUERY_FIELD (g_object_get_data (G_OBJECT (clist), "qf"));
-	user = g_object_get_data (G_OBJECT (clist), "user_func") ? TRUE : FALSE;
-	sys = g_object_get_data (G_OBJECT (clist), "sys_func") ? TRUE : FALSE;
-
-	list = qf->query->conf->srv->data_functions;
-	while (list) {
-		if ((user && (SERVER_FUNCTION (list->data)->is_user)) ||
-		    (sys && (!SERVER_FUNCTION (list->data)->is_user))) {
-			line[0] = g_strdup (SERVER_FUNCTION (list->data)->sqlname);
-			line[1] = g_strdup (SERVER_FUNCTION (list->data)->descr);
-			i = gtk_clist_append (clist, line);
-			gtk_clist_set_row_data (clist, i, list->data);
-			g_free (line[0]);
-			g_free (line[1]);
-		}
-		list = g_slist_next (list);
-	}
-}
-
-static void 
-function_selected_cb (GtkWidget * wid, gint row, gint column,
-		      GdkEventButton * event, GtkCList *clist)
-{
-	ServerFunction *func;
-	QueryField *qf;
-	
-	func = SERVER_FUNCTION (gtk_clist_get_row_data (GTK_CLIST (wid), row));
-	qf = QUERY_FIELD (g_object_get_data (G_OBJECT (clist), "qf"));
-
-	if (QF_PRIVATE_DATA(qf)->func != func) {
-		GSList *list;
-
-		query_field_deactivate (qf);
-
-		/* Function's name */
-		QF_PRIVATE_DATA(qf)->func = func;
-		
-		/* Arguments */
-		free_args (qf);
-		list = func->args;
-		while (list) {
-			QF_PRIVATE_DATA(qf)->args = g_slist_append (QF_PRIVATE_DATA(qf)->args, 
-								    g_new0 (Arg, 1));
-			list = g_slist_next (list);
-		}
-		query_field_activate (qf);
+		if (QF_PRIVATE_DATA(qf)->func != func) {
+			GSList *list;
+			
+			query_field_deactivate (qf);
+			
+			/* Function's name */
+			qf_set_func_ptr (qf, func);
+			
+			/* Arguments */
+			free_args (qf);
+			list = func->args;
+			while (list) {
+				QF_PRIVATE_DATA(qf)->args = g_slist_append (QF_PRIVATE_DATA(qf)->args, 
+									    g_new0 (Arg, 1));
+				list = g_slist_next (list);
+			}
+			query_field_activate (qf);
 #ifdef debug_signal
-		g_print (">> 'FIELD_MODIFIED' from function_selected_cb\n");
+			g_print (">> 'FIELD_MODIFIED' from function_selected_cb\n");
 #endif
-		g_signal_emit_by_name (G_OBJECT (qf), "field_modified");
+			g_signal_emit_by_name (G_OBJECT (qf), "field_modified");
 #ifdef debug_signal
-		g_print ("<< 'FIELD_MODIFIED' from function_selected_cb\n");
+			g_print ("<< 'FIELD_MODIFIED' from function_selected_cb\n");
 #endif	
+		}
 	}
 }
 
 static GtkWidget * 
-q_get_sel_widget  (QueryField *qf, GCallback callback, gpointer data)
+q_get_sel_widget  (QueryField *qf, QueryField *sel_qf, GCallback callback, gpointer data)
 {
 	GtkWidget *hbox;
 	GtkWidget *button, *label;
@@ -649,6 +520,10 @@ q_get_sel_widget  (QueryField *qf, GCallback callback, gpointer data)
 	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
 	gtk_widget_show (button);	
 
+	/* state */
+	if (sel_qf == qf) 
+		gtk_widget_set_sensitive (button, FALSE);
+
 	label = gtk_label_new (" (");
 	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
 	gtk_widget_show (label);
@@ -674,7 +549,7 @@ q_get_sel_widget  (QueryField *qf, GCallback callback, gpointer data)
 		}
 
 		/* sub widget */
-		wid = query_field_get_select_widget (QF_FUNC_ARG(list->data)->qf, callback, data);
+		wid = query_field_get_select_widget (QF_FUNC_ARG(list->data)->qf, sel_qf, callback, data);
 		gtk_box_pack_start (GTK_BOX (hbox), wid, TRUE, TRUE, 0);
 		gtk_widget_show (wid);
 
@@ -698,7 +573,7 @@ q_render_as_sql   (QueryField *qf, GSList * missing_values)
 {
 	gchar *str = NULL;
 
-	if (qf->activated) {
+	if (qf->priv->activated) {
 		GString *string;
 		GSList *list;
 		gboolean firstarg = TRUE;
@@ -715,6 +590,8 @@ q_render_as_sql   (QueryField *qf, GSList * missing_values)
 				g_string_append (string, ", ");
 
 			str2 = query_field_render_as_sql (QF_FUNC_ARG(list->data)->qf, NULL);
+			if (!str2)
+				str2 = g_strdup ("NULL");
 			g_string_append (string, str2);
 			g_free (str2);
 			list = g_slist_next (list);
@@ -739,7 +616,7 @@ q_render_as_string(QueryField *qf, GSList * missing_values)
 {
 	gchar *str = NULL;
 
-	if (qf->activated) {
+	if (qf->priv->activated) {
 		GString *string;
 		GSList *list;
 		gboolean firstarg = TRUE;
@@ -775,15 +652,18 @@ q_render_as_string(QueryField *qf, GSList * missing_values)
 static void  
 q_save_to_xml     (QueryField *qf, xmlNodePtr node)
 {
-	if (qf->activated) {
+	if (QF_PRIVATE_DATA(qf)->func || QF_PRIVATE_DATA(qf)->func_name) {
 		GSList *list;
 		gchar *str;
 		xmlNodePtr arg;
 
-		g_print("Saving field to node %p...", node);	
 
 		/* node object ref */
-		str = server_function_get_xml_id (QF_PRIVATE_DATA(qf)->func);
+		if (QF_PRIVATE_DATA(qf)->func_name)
+			str = g_strdup (QF_PRIVATE_DATA(qf)->func_name);
+		else 
+			str = server_function_get_xml_id (QF_PRIVATE_DATA(qf)->func);
+
 		xmlSetProp (node, "object", str);
 		g_free (str);
 
@@ -791,17 +671,25 @@ q_save_to_xml     (QueryField *qf, xmlNodePtr node)
 		list = QF_PRIVATE_DATA(qf)->args;
 		while (list) {
 			arg = xmlNewChild (node, NULL, "QueryFieldArg", NULL);
-			str = query_field_get_xml_id (QF_FUNC_ARG(list->data)->qf);
-			xmlSetProp (arg, "ref", str);
-			g_free (str);
+			if (QF_FUNC_ARG(list->data)->xml)
+				str = g_strdup (QF_FUNC_ARG(list->data)->xml);
+			else {
+				if (QF_FUNC_ARG(list->data)->qf)
+					str = query_field_get_xml_id (QF_FUNC_ARG(list->data)->qf);
+				else
+					str = NULL;
+			}
+
+			if (str) {
+				xmlSetProp (arg, "ref", str);
+				g_free (str);
+			}
 
 			list = g_slist_next (list);
 		}
-
-		g_print("done\n");
 	}
 	else
-		g_warning ("QueryField not activated; can't save\n");
+		g_warning ("Can't find QueryField's function; can't save\n");
 }
 
 static void        
@@ -831,15 +719,14 @@ q_load_from_xml   (QueryField *qf, xmlNodePtr node)
 		arg = node->xmlChildrenNode;
 		while (arg) {
 			if (*arg->name == 'Q') {
+				Arg *funcarg;
+				
+				funcarg = g_new0 (Arg, 1);
+				QF_PRIVATE_DATA(qf)->args = g_slist_append (QF_PRIVATE_DATA(qf)->args, funcarg);
+
 				str = xmlGetProp (arg, "ref");
-				if (str) {
-					Arg *arg;
-					
-					arg = g_new0 (Arg, 1);
-					arg->xml = str;
-					QF_PRIVATE_DATA(qf)->args = g_slist_append (QF_PRIVATE_DATA(qf)->args,
-										    arg);
-				}
+				if (str)
+					funcarg->xml = str;
 			}
 			arg = arg->next;
 		}
@@ -857,8 +744,9 @@ q_copy_other_field(QueryField *qf, QueryField *other)
 	   of QueryField it was before. This is normally done by the
 	   QueryField object before the copy */
 
-	if (QF_PRIVATE_DATA(other)->func) 
-		QF_PRIVATE_DATA(qf)->func = QF_PRIVATE_DATA(other)->func;
+	if (QF_PRIVATE_DATA(other)->func) {
+		qf_set_func_ptr (qf, QF_PRIVATE_DATA(other)->func);
+	}
 	else {
 		if (QF_PRIVATE_DATA(other)->func_name) 
 			QF_PRIVATE_DATA(qf)->func_name = g_strdup (QF_PRIVATE_DATA(other)->func_name);
@@ -882,7 +770,7 @@ q_is_equal_to (QueryField *qf, QueryField *other)
 {
 	gboolean retval = FALSE;
 
-	if (qf->activated && other->activated) {
+	if (qf->priv->activated && other->priv->activated) {
 		if (QF_PRIVATE_DATA(qf)->func == QF_PRIVATE_DATA(other)->func)
 			retval = TRUE;
 	}
@@ -897,16 +785,16 @@ q_get_monitored_objects (QueryField *qf)
 {
 	GSList *list = NULL, *args;
 
-	g_assert (qf && IS_QUERY_FIELD (qf));
+	g_return_val_if_fail (qf && IS_QUERY_FIELD (qf), NULL);
+
+	if (qf->priv->activated)
+		list = g_slist_append (NULL, QF_PRIVATE_DATA(qf)->func);
 
-	if (qf->activated)
-		list = g_slist_prepend (NULL, QF_PRIVATE_DATA(qf)->func);
 
-	g_print ("QF_PRIVATE_DATA(qf) = %p\n", QF_PRIVATE_DATA(qf));
 	args = QF_PRIVATE_DATA(qf)->args;
 	while (args) {
 		if (QF_FUNC_ARG(args->data)->qf)
-			list = g_slist_prepend (list, QF_FUNC_ARG(args->data)->qf);
+			list = g_slist_append (list, QF_FUNC_ARG(args->data)->qf);
 		args = g_slist_next (args);
 	}
 
@@ -952,6 +840,7 @@ q_replace_comp (QueryField *qf, gint ref, GObject   *old, GObject   *new)
 			arg = g_slist_nth_data (QF_PRIVATE_DATA(qf)->args, ref);
 			if (arg) {
 				query_field_deactivate (qf);
+				release_arg (qf, arg, TRUE);
 				if (arg->xml)
 					g_free (arg->xml);
 				arg->xml = NULL;
@@ -983,3 +872,15 @@ q_replace_comp (QueryField *qf, gint ref, GObject   *old, GObject   *new)
  *
  */
 
+ServerFunction  *
+query_field_func_get_function (QueryField *qf)
+{
+	g_return_val_if_fail (qf && IS_QUERY_FIELD (qf), NULL);
+	g_return_val_if_fail (qf->priv->field_type == QUERY_FIELD_FUNCTION, NULL);
+	
+	if (QF_PRIVATE_DATA(qf)->func)
+		return SERVER_FUNCTION (QF_PRIVATE_DATA(qf)->func);
+	else
+		return NULL;
+
+}
