@@ -131,15 +131,13 @@ struct _QueryClass
 	void (*field_print_modified)      (Query * q, QueryField * field);
 	void (*field_name_modified)       (Query * q, QueryField * field);
 	void (*field_alias_modified)      (Query * q, QueryField * field);
+	void (*fields_order_changed)      (Query * q);
 
 	void (*join_created)              (Query * q, QueryJoin * join);
 	void (*join_dropped)              (Query * q, QueryJoin * join);
 	void (*join_modified)             (Query * q, QueryJoin * join);
 
-	void (*cond_created)             (Query * q, QueryCond * qcond);
-	void (*cond_dropped)             (Query * q, QueryCond * qcond);
-	void (*cond_modified)            (Query * q, QueryCond * qcond);
-	void (*cond_moved)               (Query * q, QueryCond * qcond);
+	void (*cond_changed)              (Query * q);
 
 	void (*env_created)               (Query * q, GObject   *env);
 	void (*env_dropped)               (Query * q, GObject   *env);
@@ -158,9 +156,9 @@ struct _QueryClass
 /* 
  * Object's management
  */
-guint       query_get_type             (void);
-GObject    *query_new                  (const gchar * name, Query *parent_query, ConfManager * conf);
-GObject    *query_new_copy             (Query * q);
+guint       query_get_type                 (void);
+GObject    *query_new                      (const gchar * name, Query *parent_query, ConfManager * conf);
+GObject    *query_new_copy                 (Query * q);
 
 
 /*
@@ -168,84 +166,85 @@ GObject    *query_new_copy             (Query * q);
  */
 
 /* if NULL passed as gchar*, nothing happens for that gchar* */
-void        query_set_name             (Query * q, const gchar * name, const gchar * descr);
-void        query_set_query_type       (Query * q, QueryType type);
+void        query_set_name                 (Query * q, const gchar * name, const gchar * descr);
+void        query_set_query_type           (Query * q, QueryType type);
 /* sets the text SQL and toggles the query type */
-void        query_set_text_sql         (Query * q, gchar * sql);
+void        query_set_text_sql             (Query * q, gchar * sql);
 
 
 /*
  * XML interractions
  */
-gchar      *query_get_xml_id           (Query *q);
-Query      *query_find_from_xml_name   (ConfManager * conf, Query * start_query, gchar *xmlname);
-Query      *query_build_from_xml_tree  (ConfManager * conf, xmlNodePtr node, Query *parent);
-void        query_build_xml_tree       (Query * q, xmlNodePtr toptree, ConfManager * conf);
+gchar      *query_get_xml_id               (Query *q);
+Query      *query_find_from_xml_name       (ConfManager * conf, Query * start_query, gchar *xmlname);
+Query      *query_build_from_xml_tree      (ConfManager * conf, xmlNodePtr node, Query *parent);
+void        query_build_xml_tree           (Query * q, xmlNodePtr toptree, ConfManager * conf);
 
 /* 
  * Contents manipulation
  */
-void        query_add_sub_query        (Query *q, Query *sibling, Query *sub_query);
-void        query_del_sub_query        (Query *q, Query *sub_query);
+void        query_add_sub_query            (Query *q, Query *sibling, Query *sub_query);
+void        query_del_sub_query            (Query *q, Query *sub_query);
 
-void        query_add_view             (Query *q, QueryView *qv);
-QueryView  *query_add_view_with_obj    (Query *q, GObject   *obj);
-void        query_del_view             (Query *q, QueryView *qv); /* qv gets free'd */
+void        query_add_view                 (Query *q, QueryView *qv);
+QueryView  *query_add_view_with_obj        (Query *q, GObject   *obj);
+void        query_del_view                 (Query *q, QueryView *qv); /* qv gets free'd */
 
-void        query_add_join	       (Query *q, QueryView *v1, QueryField *f1, QueryView  *v2, QueryField *f2);
-void        query_add_join_and_fields  (Query *q, QueryView *v1, GObject *f1, QueryView  *v2, GObject *f2);
-void        query_del_join             (Query * q, QueryJoin *join);
+void        query_add_join	           (Query *q, QueryView *v1, QueryField *f1, QueryView  *v2, QueryField *f2);
+void        query_add_join_and_fields      (Query *q, QueryView *v1, GObject *f1, QueryView  *v2, GObject *f2);
+void        query_del_join                 (Query * q, QueryJoin *join);
 
 
-guint       query_get_unique_field_id  (Query *q, guint proposed_id);
-void        query_add_field            (Query *q, QueryField *field);
-void        query_add_field_before     (Query *q, QueryField *field, QueryField *sibling);
-void        query_del_field            (Query *q, QueryField *field);
-void        query_swap_fields          (Query * q, QueryField *f1, QueryField *f2);
+guint       query_get_unique_field_id      (Query *q, guint proposed_id);
+void        query_add_field                (Query *q, QueryField *field);
+void        query_add_field_before         (Query *q, QueryField *field, QueryField *sibling);
+void        query_del_field                (Query *q, QueryField *field);
+void        query_swap_fields              (Query * q, QueryField *f1, QueryField *f2);
 
-void        query_add_cond             (Query * q, QueryCond *cond, QueryCond * parent, QueryCond * sibling);
-void        query_del_cond             (Query * q, QueryCond *cond);
-void        query_move_cond            (Query * q, QueryCond * cond, QueryCond * to_parent, QueryCond * to_sibling);
+void        query_add_cond                 (Query * q, QueryCond *cond, QueryCond * parent, QueryCond * sibling);
+void        query_del_cond                 (Query * q, QueryCond *cond);
+void        query_move_cond                (Query * q, QueryCond * cond, 
+					    QueryCond * to_parent, QueryCond * to_sibling);
 
-void        query_add_env              (Query * q, GObject *env);
-void        query_del_env              (Query * q, GObject *env);
+void        query_add_env                  (Query * q, GObject *env);
+void        query_del_env                  (Query * q, GObject *env);
 
 
 /*
  * Getting and finding information
  */
-gchar      *query_get_select_query     (Query * q, GSList * missing_values);
+gchar      *query_get_select_query         (Query * q, GSList * missing_values);
 
-gboolean    query_is_query_compatible  (Query *q, Query *query_to_add);
-gboolean    query_fields_activated     (Query *q);
+gboolean    query_is_query_compatible      (Query *q, Query *query_to_add);
+gboolean    query_fields_activated         (Query *q);
 GSList     *query_get_missing_value_fields (Query *q); /* lists only QF of type VALUE */
 
-QueryView  *query_get_view_by_xmlid    (Query * q, const gchar * xmlid);
+QueryView  *query_get_view_by_xmlid        (Query * q, const gchar * xmlid);
 
 /* returns the QueryField of the query given the column number it occupies
    int the ServerResultset of the SELECT row */
-QueryField *query_get_field_by_pos     (Query * q, gint pos);       /* NULL if error */
-gint        query_get_pos_by_field     (Query * q, QueryField *qf); /* -1 if error */
+QueryField *query_get_field_by_pos         (Query * q, gint pos);       /* NULL if error */
+gint        query_get_pos_by_field         (Query * q, QueryField *qf); /* -1 if error */
 
-QueryField *query_get_field_by_name    (Query * q, gchar * name);
-QueryField *query_get_field_by_xmlid   (Query * q, gchar * xmlid);
-QueryField *query_get_field_by_id      (Query * q, guint id);
-QueryField *query_get_field_by_dbfield    (Query * q, QueryView *qv, DbField *field);
-QueryField *query_get_field_by_queryfield (Query * q, QueryView *qv, QueryField *field); /* field is of another query */
+QueryField *query_get_field_by_name        (Query * q, gchar * name);
+QueryField *query_get_field_by_xmlid       (Query * q, gchar * xmlid);
+QueryField *query_get_field_by_id          (Query * q, guint id);
+QueryField *query_get_field_by_dbfield     (Query * q, QueryView *qv, DbField *field);
+QueryField *query_get_field_by_queryfield  (Query * q, QueryView *qv, QueryField *field); /* field is of another query */
 
 
 /* tells if a particular field appears in the print list of the Query:
    returns -1 if not present and >=0 otherwise, the number being the column
    number of the ServerResultset of the SELECT query */
-gint        query_is_table_field_printed  (Query * q, DbField * field);
+gint        query_is_table_field_printed   (Query * q, DbField * field);
 
 /* Finds all the equi QueryJoin which have the qf QueryField part of the condition and
 where the cardinality is N to qf and 1 to the other end. qf MUST be of field_type
 QUERY_FIELD_FIELD */
-GSList     *query_find_equi_join_with_end (Query * q, QueryField *qf);
+GSList     *query_find_equi_join_with_end  (Query * q, QueryField *qf);
 
 #ifdef debug
-void        query_dump_contents           (Query *q);
+void        query_dump_contents            (Query *q);
 #endif
 
 
@@ -418,11 +417,11 @@ GObject       *query_field_new_copy_all      (QueryField *field, GSList **list);
 /*
  * Contents modifications
  */
-void           query_field_set_name          (QueryField * qf, const gchar * name);
-void           query_field_set_alias         (QueryField * qf, const gchar * alias);
-void           query_field_set_is_printed    (QueryField * qf, gboolean is_printed);
-void           query_field_set_is_hidden     (QueryField * qf, gboolean is_hidden);
-void           query_field_set_id            (QueryField * qf, guint id);
+void           query_field_set_name          (QueryField *qf, const gchar * name);
+void           query_field_set_alias         (QueryField *qf, const gchar * alias);
+void           query_field_set_is_printed    (QueryField *qf, gboolean is_printed);
+void           query_field_set_is_hidden     (QueryField *qf, gboolean is_hidden);
+void           query_field_set_id            (QueryField *qf, guint id);
 /* copies the contents of a QueryField into another alredy existing */
 void           query_field_copy_object       (QueryField *dest, QueryField *orig);
 
@@ -430,10 +429,10 @@ void           query_field_copy_object       (QueryField *dest, QueryField *orig
 /* 
  * Activation
  */
-void           query_field_deactivate        (QueryField * qf);
-void           query_field_activate          (QueryField * qf);
-gboolean       query_field_get_activated     (QueryField * qf);
-
+void           query_field_deactivate        (QueryField *qf);
+void           query_field_activate          (QueryField *qf);
+gboolean       query_field_get_activated     (QueryField *qf);
+void           query_field_nullify           (QueryField *qf);
 
 /*
  * Getting information / using the QF
@@ -510,11 +509,14 @@ gint              query_field_allfields_get_span      (QueryField *qf);
 
 /* Representation of a value */
 ServerDataType  * query_field_value_get_data_type     (QueryField *qf);
-const GdaValue  * query_field_value_get_default_value (QueryField *qf);
+const GdaValue  * query_field_value_get_value         (QueryField *qf);
 gboolean          query_field_value_is_parameter      (QueryField *qf);
 void              query_field_value_set_value         (QueryField *qf, const GdaValue *value);
 
+
 /* Representation of a function */
+ServerFunction  * query_field_func_get_function       (QueryField *qf);
+
 
 /* Representation of a whole query */
 Query           * query_field_query_get_query         (QueryField *qf);
@@ -739,7 +741,7 @@ QueryField      *query_cond_get_op_right2    (QueryCond *qc);
 
 gchar          * query_cond_render_as_sql    (QueryCond *qc, GSList * missing_values);
 xmlNodePtr       query_cond_render_as_xml    (QueryCond *qc, GSList * missing_values);
-
+const gchar *    query_cond_render_op_sql    (QueryCond *qc);
 
 /*
  * XML manipulations
